000100 IDENTIFICATION DIVISION.
000200 PROGRAM-ID. MMS1AB.
000201*       RELEASE DATE=08/24/01.        REVISION LEVEL=08.06      * ++++++++
000202***************************************************************** ++++++++
000203*WARNING.  THIS COMPUTER PROGRAM IS PROTECTED BY COPYRIGHT LAW  * ++++++++
000204*AND INTERNATIONAL TREATIES.  UNAUTHORIZED REPRODUCTION OR      * ++++++++
000205*DISTRIBUTION OF THIS PROGRAM, OR ANY PORTION OF IT, MAY RESULT * ++++++++
000206*IN SEVERE CIVIL AND CRIMINAL PENATLIES, AND WILL BE PROSECUTED * ++++++++
000207*TO THE MAXIMUM EXTENT POSSIBLE UNDER THE LAW.                  * ++++++++
000208*COPYRIGHT 1999 PAYSYS INTERNATIONAL, INC.  ALL RIGHTS RESERVED.* ++++++++
000209***************************************************************** ++++++++
000300******************************************************************
000400*                                                                *
000500*    ----------------MERCHANT BANKCARD SYSTEM----------------    *
000600*                                                                *
000700*           MMS1AB - MMS1 FILE COMMON BATCH I/O ROUTINE          *
000800*                                                                *
000900*                    VARIABLE LENGTH VSAM FILE                   *
001000*                                                                *
001100*    ---------------ON-LINE STATEMENT FILE 1---------------      *
001200*                                                                *
001300******************************************************************
001400*                                                                *
001500*    COPYBOOKS WHICH MUST APPEAR IN YOUR BATCH PROGRAM:          *
001600*                                                                *
001700*    (WS) MB00RB - BATCH REQUEST BLOCK                           *
001800*    (WS) MMS1IO - I/O AREA                                      *
001900*    (WS) SS02WS - STANDARD ABEND ROUTINE WORK AREA              *
002000*    (PD) MB00PD - PROCEDURE DIVISION CALL STATEMENT             *
002100*                                                                *
002200******************************************************************
002300*                                                                *
002400*    TO USE THIS ROUTINE.........                                *
002500*                                                                *
002600* 1. HOUSEKEEPING DUTIES:                                        *
002700*                                                                *
002800*    A. IN THE HOUSEKEEPING SECTION OF YOUR PROGRAM, MOVE THE    *
002900*       NAME OF THE PROGRAM USING THIS ROUTINE TO THE FIELD      *
003000*       'SS02WS-ABEND-PROG-ID'. THIS FIELD IS USED AS PART OF THE*
003100*       STANDARD ABEND MESSAGES IF A PROGRAM ABORTS WITHIN THIS  *
003200*       ACCESS ROUTINE.  IF THE FIELD 'SS02WS-ABEND-PROG-ID' IS  *
003300*       LEFT BLANK, THIS ROUTINE WILL FORCE AN ABEND SITUATION.  *
003400*                                                                *
003500*    B. BEFORE PERFORMING THE ACCESS ROUTINE, MOVE A LITERAL TO  *
003600*       THE FIELD 'SS02WS-ABEND-LOCATION'. THIS FIELD CAN BE USED*
003700*       TO INDICATE WHERE THE ROUTINE IS BEING 'FROM' IN CASE    *
003800*       OF ABNORMAL TERMINATION. (E.G. DEBUGGING ASSISTANCE)     *
003900*                                                                *
004000*    C. SET THE RETURN CODES WHICH YOU WANT, FOR THE DURATION    *
004100*       OF THIS PROGRAM, TO BE CONSIDERED VALID IN THE FIELDS    *
004200*       'MMS1RB-PERM-STATUS-1 TO -5'. THESE FIELDS ARE INITIAL-  *
004300*       IZED TO ZERO (AS ZERO IS ALWAYS AN ACCEPTABLE RETURN     *
004400*       CODE).  THIS ROUTINE WILL (IN ALL I-O REQUESTS) SUCCESS- *
004500*       FULLY SATISFY YOUR PERFORM REQUEST IF ANY OF THE REQUESTS*
004600*       ENCOUNTER ANY OF THE PERMANENT STATUS CODES 1-5.         *
004700*                                                                *
004800* 2. SET ONE OF THE FOLLOWING CONDITION NAMES TO 'TRUE' TO SET   *
004900*    THE FILE REQUEST:                                           *
005000*                                                                *
005100*        CONDITION NAME                EXPLANATION               *
005200*   *----------------------***---------------------------------* *
005300*                                                                *
005400*    MMS1RB-OPEN-INPUT      - OPEN THE FILE AS INPUT WITH        *
005500*                             SEQUENTIAL ACCESS.                 *
005600*    MMS1RB-OPEN-OUTPUT     - OPEN THE FILE AS OUTPUT WITH       *
005700*                             SEQUENTIAL ACCESS.                 *
005800*    MMS1RB-OPEN-IO         - OPEN THE FILE AS I-O WITH          *
005900*                             SEQUENTIAL ACCESS.                 *
006000*    MMS1RB-OPEN-INPUT-DYN  - OPEN THE FILE AS INPUT WITH        *
006100*                             DYNAMIC ACCESS.                    *
006200*    MMS1RB-OPEN-OUTPUT-DYN - OPEN THE FILE AS OUTPUT WITH       *
006300*                             DYNAMIC ACCESS.                    *
006400*    MMS1RB-OPEN-IO-DYN     - OPEN THE FILE AS I-O WITH          *
006500*                             DYNAMIC ACCESS.                    *
006600*    MMS1RB-OPEN-INPUT-RDM  - OPEN THE FILE AS INPUT WITH        *
006700*                             RANDOM ACCESS.                     *
006800*    MMS1RB-OPEN-OUTPUT-RDM - OPEN THE FILE AS OUTPUT WITH       *
006900*                             RANDOM ACCESS.                     *
007000*    MMS1RB-OPEN-IO-RDM     - OPEN THE FILE AS I-O WITH          *
007100*                             RANDOM ACCESS.                     *
007200*    MMS1RB-START-GTE       - START THE FILE WIOTH A KEY         *
007300*                             > OR            = THE        KEY   *
007400*                             CONTAINED IN 'MMS1IO-KEY'.         *
007500*    MMS1RB-START-EQUAL     - START THE FILE WITH A KEY =        *
007600*                             TO THE KEY CONTAINED IN            *
007700*                             'MMS1IO-KEY'.                      *
007800*    MMS1RB-READ-SEQ        - READ THE NEXT SEQUENTIAL RECORD    *
007900*                             IN THE FILE.                       *
008000*    MMS1RB-READ-RANDOM     - READ THE RECORD IN THE FILE        *
008100*                             CORRESPONDING TO THE KEY CONTAINED *
008200*                             IN 'MMS1IO-KEY'.                   *
008300*    MMS1RB-WRITE           - WRITE A RECORD (THE RECORD MUST    *
008400*                             BE IN 'MMS1IO-RECORD').            *
008500*    MMS1RB-REWRITE         - REWRITE A RECORD (THE RECORD MUST  *
008600*                             BE IN 'MMS1IO-RECORD').            *
008700*    MMS1RB-DELETE          - DELETE THE RECORD IN THE FILE      *
008800*                             CORRESPONDING TO THE KEY CONTAINED *
008900*                             IN 'MMS1IO-KEY'.                   *
009000*    MMS1RB-CLOSE           - CLOSE THE FILE.                    *
009100*                                                                *
009200* 3. SET ONE OR BOTH OF THE 'TEMPORARY' RETURN CODE FIELDS       *
009300*    'MMS1RB-TEMP-STATUS-1 AND -2'.  THESE TWO RETURN CODES      *
009400*    ARE TO BE USED ON A PER USE BASIS OF THIS ROUTINE.          *
009500*    THESE TWO RETURN CODES ARE INITIALIZED TO ZERO AND ARE      *
009600*    BOTH RESET TO ZERO AFTER EACH CALL OF THIS PROGRAM.         *
009700*    THE TEMPORARY RETURN CODE FIELDS ARE CHECKED FIRST AND      *
009800*    THEN THE PERMANENT RETURN CODES (POINT 1.C ABOVE) WILL      *
009900*    BE CHECKED.                                                 *
010000*                                                                *
010100* 4. PERFORM MB00PD-MMS1-ACCESS THRU MB00PD-MMS1-ACCESS-EXIT.    *
010200*                                                                *
010300*    A. YOU MUST HANDLE MANUALLY ANY RETURN CODE (EITHER TEMP    *
010400*       OR PERM) THAT YOU REQUESTED, AND IF NEEDED, YOUR OWN     *
010500*       ABEND REQUEST.                                           *
010600*                                                                *
010700*    B. ALL ACCESS ROUTINES MOVE THE FILE STATUS TO THE FIELD    *
010800*       SS02WS-FILE-STATUS IN COPYBOOK SS02WS. USE THE '88' LEVEL*
010900*       NAMES TO TEST YOUR MANUAL STATUS CHECKS.  ALSO IN COPY   *
011000*       BOOK SS02WS, USE THE 'WS-' FILE STATUS DATA NAMES WHEN   *
011100*       SETTING THE PERM OR TEMP FILE STATUS CODES PRIOR TO      *
011200*       PERFORMING THE ACCESS ROUTINE.  FOR VSAM FILES, THE VSAM *
011300*       RETURN CODE IS MOVED TO THE FIELD 'SS02WS-FILE-RETURN' IN*
011400*       COPYBOOK SS02WS.                                         *
011500*                                                                *
011600******************************************************************
011700/
011800 ENVIRONMENT DIVISION.
011900*
012000 CONFIGURATION SECTION.
012100 SOURCE-COMPUTER.            IBM-4341.
012200 OBJECT-COMPUTER.            IBM-4341.
012300*
012400 INPUT-OUTPUT SECTION.
012500*
012600 FILE-CONTROL.
012700*
012800     SELECT  MMS1-FILE-SEQ
012900             ASSIGN TO MMS1
013000             ORGANIZATION IS INDEXED
013100             ACCESS       IS SEQUENTIAL
013200             FILE STATUS  IS WS-STATUS
013300                             WS-RETURN
013400             RECORD KEY   IS MMS1-SEQ-KEY.
013500*
013600     SELECT  MMS1-FILE-DYN
013700             ASSIGN TO MMS1
013800             ORGANIZATION IS INDEXED
013900             ACCESS       IS DYNAMIC
014000             FILE STATUS  IS WS-STATUS
014100                             WS-RETURN
014200             RECORD KEY   IS MMS1-DYN-KEY.
014300*
014400     SELECT  MMS1-FILE-RDM
014500             ASSIGN TO MMS1
014600             ORGANIZATION IS INDEXED
014700             ACCESS       IS RANDOM
014800             FILE STATUS  IS WS-STATUS
014900                             WS-RETURN
015000             RECORD KEY   IS MMS1-RDM-KEY.
015100*
015200 I-O-CONTROL.
015300*
015400     SAME AREA FOR
015500         MMS1-FILE-SEQ
015600         MMS1-FILE-DYN
015700         MMS1-FILE-RDM.
015800/
015900 DATA DIVISION.
016000*
016100 FILE SECTION.
016200*
016300***********************
016400*    SEQUENTIAL FD    *
016500***********************
016600*
016700 FD  MMS1-FILE-SEQ
016800     RECORD CONTAINS 27 TO 377 CHARACTERS.
016900*
017000 01  MMS1-SEQ-RECORD.
017100     03  MMS1-SEQ-KEY        PIC  X(25).
017200     03  MMS1-REC-LENGTH     PIC  S9(04)     BINARY.
017300     03  MMS1-SEQ-DATA       PIC  X(01)      OCCURS
017400                                             0 TO 350 TIMES
017500                                             DEPENDING ON
017600                                             WS-DATA-LENGTH.
017700*
017800********************
017900*    DYNAMIC FD    *
018000********************
018100*
018200 FD  MMS1-FILE-DYN
018300     RECORD CONTAINS 27 TO 377 CHARACTERS.
018400*
018500 01  MMS1-DYN-RECORD.
018600     03  MMS1-DYN-KEY        PIC  X(25).
018700     03  MMS1-REC-LENGTH     PIC  S9(04)     BINARY.
018800     03  MMS1-DYN-DATA       PIC  X(01)      OCCURS
018900                                             0 TO 350 TIMES
019000                                             DEPENDING ON
019100                                             WS-DATA-LENGTH.
019200*
019300*******************
019400*    RANDOM FD    *
019500*******************
019600*
019700 FD  MMS1-FILE-RDM
019800     RECORD CONTAINS 27 TO 377 CHARACTERS.
019900*
020000 01  MMS1-RDM-RECORD.
020100     03  MMS1-RDM-KEY        PIC  X(25).
020200     03  MMS1-REC-LENGTH     PIC  S9(04)     BINARY.
020300     03  MMS1-RDM-DATA       PIC  X(01)      OCCURS
020400                                             0 TO 350 TIMES
020500                                             DEPENDING ON
020600                                             WS-DATA-LENGTH.
020700/
020800 WORKING-STORAGE SECTION.
020900*
021000 01  FILLER                  PIC  X(12)      VALUE '** WS-BEG **'.
021100*
021200 01  FILLER                  PIC  X(24)      VALUE
021300     'ACCESS METHOD INDICATOR:'.
021400*
021500 01  WS-ACCESS-METHOD        PIC  X(01)      VALUE SPACE.
021600     88  ACCESS-METHOD-NOT-SET               VALUE SPACE.
021700     88  ACCESS-IS-SEQUENTIAL                VALUE 'S'.
021800     88  ACCESS-IS-DYNAMIC                   VALUE 'D'.
021900     88  ACCESS-IS-RANDOM                    VALUE 'R'.
022000*
022100 01  FILLER                  PIC  X(12)      VALUE
022200     'FILE STATUS:'.
022300*
022400 01  WS-STATUS               PIC  X(02)      VALUE ZERO.
022500*
022600 01  FILLER                  PIC  X(16)      VALUE
022700     'EXTENDED RETURN:'.
022800*
022900 01  WS-RETURN.
023000     03  WS-RETURN-CODE      PIC 9(02)       BINARY
023100                                             VALUE ZERO.
023200     03  WS-FUNCTION-CODE    PIC 9(02)       BINARY
023300                                             VALUE ZERO.
023400     03  WS-FEEDBACK-CODE    PIC 9(02)       BINARY
023500                                             VALUE ZERO.
023600*
023700 01  FILLER                  PIC  X(12)      VALUE
023800     'I-O REQUEST:'.
023900*
024000 01  WS-IO-REQUEST           PIC  9(02)      VALUE ZERO.
024100*
024200 01  FILLER                  PIC  X(12)      VALUE
024300     'DATA LENGTH:'.
024400*
024500 01  WS-DATA-LENGTH          PIC S9(04)      BINARY
024600                                             VALUE ZERO.
024700     88  WS-MAXIMUM-DATA-LENGTH              VALUE +350.
024800*
024900 01  FILLER                  PIC  X(28)      VALUE
025000     'PROCESSING SUMMARY COUNTERS:'.
025100*
025200 01  WS-SUMMARY-COUNTERS.
025300     03  WS-SC-READS         PIC S9(09)      PACKED-DECIMAL
025400                                             VALUE ZERO.
025500     03  WS-SC-WRITES        PIC S9(09)      PACKED-DECIMAL
025600                                             VALUE ZERO.
025700     03  WS-SC-REWRITES      PIC S9(09)      PACKED-DECIMAL
025800                                             VALUE ZERO.
025900     03  WS-SC-DELETES       PIC S9(09)      PACKED-DECIMAL
026000                                             VALUE ZERO.
026100*
026200 01  FILLER                  PIC  X(27)      VALUE
026300     'PROCESSING SUMMARY MESSAGE:'.
026400*
026500 01  WS-SUMMARY-MESSAGE.
026600     03  FILLER              PIC  X(10)      VALUE '(PGMSTAT) '.
026700     03  WS-SM-PROG-ID       PIC  X(06)      VALUE SPACES.
026800     03  FILLER              PIC  X(03)      VALUE ' - '.
026900     03  WS-SM-FILE-ID       PIC  X(04)      VALUE 'MMS1'.
027000     03  FILLER              PIC  X(11)      VALUE ' - OPENED: '.
027100     03  WS-SM-ACCESS        PIC  X(03)      VALUE SPACES.
027200     03  FILLER              PIC  X(01)      VALUE SPACE.
027300     03  WS-SM-OPENED        PIC  X(06)      VALUE SPACES.
027400     03  FILLER              PIC  X(01)      VALUE SPACE.
027500     03  WS-SM-READS         PIC  ZZZZZZZZ9.
027600     03  FILLER              PIC  X(08)      VALUE ' READS, '.
027700     03  WS-SM-WRITES        PIC  ZZZZZZZZ9.
027800     03  FILLER              PIC  X(09)      VALUE ' WRITES, '.
027900     03  WS-SM-REWRITES      PIC  ZZZZZZZZ9.
028000     03  FILLER              PIC  X(11)      VALUE ' REWRITES, '.
028100     03  WS-SM-DELETES       PIC  ZZZZZZZZ9.
028200     03  FILLER              PIC  X(08)      VALUE ' DELETES'.
028300/
028400     COPY SS00WS.
028500*
028600 01  FILLER                  PIC  X(12)      VALUE '** WS-END **'.
028700/
028800 LINKAGE SECTION.
028900*
029000****************************
029100*    FILE REQUEST BLOCK    *
029200****************************
029300*
029400     COPY MB00RB REPLACING ==:MB00:== BY ==MMS1==.
029500*
029600******************
029700*    I/O AREA    *
029800******************
029900*
030000     COPY MMS1IO REPLACING ==:MMS1:== BY ==MMS1==.
030100*
030200*********************************
030300*    ABEND ROUTINE WORK AREA    *
030400*********************************
030500*
030600     COPY SS02WS.
030700/
030800 PROCEDURE DIVISION          USING MMS1RB-WORK-AREA,
030900                                   MMS1IO-RECORD,
031000                                   SS02WS-ABEND-WORK-AREA.
031100*
031200 A000-MAIN-LINE-SECTION.
031300*
031400*
031500***************************************************
031600*    SET THE ACCESS METHOD AND ABEND WORK AREA    *
031700*    IF THIS IS THE FIRST CALL OF THIS PROGRAM    *
031800***************************************************
031900*
032000     IF  ACCESS-METHOD-NOT-SET
032100         PERFORM Z000-PROGRAM-INITIALIZATION
032200            THRU Z099-PI-EXIT.
032300*
032400**************************************************************
032500*    PROCESS THE FILE DEPENDING OF THE PROCESSING OPTION.    *
032600**************************************************************
032700*
032800     IF  ACCESS-IS-SEQUENTIAL
032900         PERFORM S000-SEQUENTIAL-ACCESS
033000            THRU S999-SA-EXIT
033100     ELSE
033200         IF  ACCESS-IS-DYNAMIC
033300             PERFORM D000-DYNAMIC-ACCESS
033400                THRU D999-DA-EXIT
033500         ELSE
033600             PERFORM R000-RANDOM-ACCESS
033700                THRU R999-RA-EXIT.
033800*
033900*****************************************************
034000*    MOVE THE STATUS CODE AND RETURN CODE TO        *
034100*    THEIR RESPECTIVE FIELDS IN COPYBOOK SS02WS.    *
034200*****************************************************
034300*
034400     MOVE MMS1RB-STATUS          TO SS02WS-FILE-STATUS.
034500     MOVE MMS1RB-RETURN          TO SS02WS-FILE-RETURN.
034600*
034700 A999-GOBACK.
034800*
034900     GOBACK.
035000/
035100 D000-DYNAMIC-ACCESS.
035200*
035300     GO TO D100-DA-INVALID-REQUEST
035400           D100-DA-INVALID-REQUEST
035500           D100-DA-INVALID-REQUEST
035600           D110-DA-OPEN-INPUT
035700           D120-DA-OPEN-OUTPUT
035800           D130-DA-OPEN-I-O
035900           D100-DA-INVALID-REQUEST
036000           D100-DA-INVALID-REQUEST
036100           D100-DA-INVALID-REQUEST
036200           D140-DA-START-GTE
036300           D150-DA-START-EQUAL
036400           D160-DA-READ-SEQ
036500           D170-DA-READ-RANDOM
036600           D180-DA-WRITE
036700           D190-DA-REWRITE
036800           D200-DA-DELETE
036900           D210-DA-CLOSE
037000           DEPENDING ON MMS1RB-IO-REQUEST.
037100*
037200 D100-DA-INVALID-REQUEST.
037300*
037400     MOVE MMS1RB-IO-REQUEST      TO SS02WS-ABEND-REQUEST.
037500     MOVE '- BAD I-O REQ - MMS1' TO SS02WS-ABENDMSG8.
037600     GO TO Z950-ABORT.
037700*
037800 D110-DA-OPEN-INPUT.
037900*
038000     OPEN INPUT MMS1-FILE-DYN.
038100*
038200     IF  WS-STATUS IS EQUAL TO '97'
038300         MOVE '97'               TO MMS1RB-OPEN-STATUS
038400         MOVE LOW-VALUES         TO WS-RETURN
038500         MOVE ZEROS              TO WS-STATUS.
038600*
038700     MOVE 'DYN'                  TO WS-SM-ACCESS.
038800     MOVE 'INPUT'                TO WS-SM-OPENED.
038900     MOVE '- OPEN  ERROR - MMS1' TO SS02WS-ABENDMSG8.
039000     GO TO D900-DA-CHECK-STATUS.
039100*
039200 D120-DA-OPEN-OUTPUT.
039300*
039400     OPEN OUTPUT MMS1-FILE-DYN.
039500*
039600     IF  WS-STATUS IS EQUAL TO '97'
039700         MOVE '97'               TO MMS1RB-OPEN-STATUS
039800         MOVE LOW-VALUES         TO WS-RETURN
039900         MOVE ZEROS              TO WS-STATUS.
040000*
040100     MOVE 'DYN'                  TO WS-SM-ACCESS.
040200     MOVE 'OUTPUT'               TO WS-SM-OPENED.
040300     MOVE '- OPEN  ERROR - MMS1' TO SS02WS-ABENDMSG8.
040400     GO TO D900-DA-CHECK-STATUS.
040500*
040600 D130-DA-OPEN-I-O.
040700*
040800     OPEN I-O MMS1-FILE-DYN.
040900*
041000     MOVE 'DYN'                  TO WS-SM-ACCESS.
041100     MOVE 'I-O'                  TO WS-SM-OPENED.
041200     MOVE '- OPEN  ERROR - MMS1' TO SS02WS-ABENDMSG8.
041300     GO TO D900-DA-CHECK-STATUS.
041400*
041500 D140-DA-START-GTE.
041600*
041700     MOVE MMS1IO-KEY             TO MMS1-DYN-KEY.
041800     START MMS1-FILE-DYN
041900           KEY IS NOT LESS THAN MMS1-DYN-KEY.
042000*
042100     MOVE '- START ERROR - MMS1' TO SS02WS-ABENDMSG8.
042200     GO TO D900-DA-CHECK-STATUS.
042300*
042400 D150-DA-START-EQUAL.
042500*
042600     MOVE MMS1IO-KEY             TO MMS1-DYN-KEY.
042700     START MMS1-FILE-DYN
042800           KEY IS EQUAL TO MMS1-DYN-KEY.
042900*
043000     MOVE '- START ERROR - MMS1' TO SS02WS-ABENDMSG8.
043100     GO TO D900-DA-CHECK-STATUS.
043200*
043300 D160-DA-READ-SEQ.
043400*
043500     SET WS-MAXIMUM-DATA-LENGTH  TO TRUE.
043600     READ MMS1-FILE-DYN NEXT RECORD
043700                                 INTO MMS1IO-RECORD.
043800*
043900     ADD 1                       TO WS-SC-READS.
044000     MOVE '- READ  ERROR - MMS1' TO SS02WS-ABENDMSG8.
044100     GO TO D900-DA-CHECK-STATUS.
044200*
044300 D170-DA-READ-RANDOM.
044400*
044500     SET WS-MAXIMUM-DATA-LENGTH  TO TRUE.
044600     MOVE MMS1IO-KEY             TO MMS1-DYN-KEY.
044700     READ MMS1-FILE-DYN          INTO MMS1IO-RECORD.
044800*
044900     ADD 1                       TO WS-SC-READS.
045000     MOVE '- READ  ERROR - MMS1' TO SS02WS-ABENDMSG8.
045100     GO TO D900-DA-CHECK-STATUS.
045200*
045300 D180-DA-WRITE.
045400*
045500     MOVE MMS1IO-REC-LENGTH      TO WS-DATA-LENGTH.
045600*
045700     WRITE MMS1-DYN-RECORD       FROM MMS1IO-RECORD.
045800*
045900     ADD 1                       TO WS-SC-WRITES.
046000     MOVE '- WRITE ERROR - MMS1' TO SS02WS-ABENDMSG8.
046100     GO TO D900-DA-CHECK-STATUS.
046200*
046300 D190-DA-REWRITE.
046400*
046500     MOVE MMS1IO-REC-LENGTH      TO WS-DATA-LENGTH.
046600*
046700     REWRITE MMS1-DYN-RECORD     FROM MMS1IO-RECORD.
046800*
046900     ADD 1                       TO WS-SC-REWRITES.
047000     MOVE '- REWRT ERROR - MMS1' TO SS02WS-ABENDMSG8.
047100     GO TO D900-DA-CHECK-STATUS.
047200*
047300 D200-DA-DELETE.
047400*
047500     MOVE MMS1IO-KEY             TO MMS1-DYN-KEY.
047600     DELETE MMS1-FILE-DYN RECORD.
047700*
047800     ADD 1                       TO WS-SC-DELETES.
047900     MOVE '- DELET ERROR - MMS1' TO SS02WS-ABENDMSG8.
048000     GO TO D900-DA-CHECK-STATUS.
048100*
048200 D210-DA-CLOSE.
048300*
048400     MOVE SPACE                  TO WS-ACCESS-METHOD.
048500     CLOSE  MMS1-FILE-DYN.
048600*
048700     PERFORM Z100-DISPLAY-THE-SUMMARY
048800        THRU Z199-DTS-EXIT.
048900*
049000     MOVE '- CLOSE ERROR - MMS1' TO SS02WS-ABENDMSG8.
049100     GO TO D900-DA-CHECK-STATUS.
049200*
049300 D900-DA-CHECK-STATUS.
049400*
049500     MOVE WS-STATUS              TO MMS1RB-STATUS.
049600     MOVE WS-RETURN              TO MMS1RB-RETURN.
049700*
049800     IF  MMS1RB-STATUS IS EQUAL  TO MMS1RB-TEMP-STATUS-1   OR
049900         MMS1RB-STATUS IS EQUAL  TO MMS1RB-TEMP-STATUS-2   OR
050000         MMS1RB-STATUS IS EQUAL  TO MMS1RB-PERM-STATUS-1   OR
050100         MMS1RB-STATUS IS EQUAL  TO MMS1RB-PERM-STATUS-2   OR
050200         MMS1RB-STATUS IS EQUAL  TO MMS1RB-PERM-STATUS-3   OR
050300         MMS1RB-STATUS IS EQUAL  TO MMS1RB-PERM-STATUS-4   OR
050400         MMS1RB-STATUS IS EQUAL  TO MMS1RB-PERM-STATUS-5
050500         MOVE ZERO               TO MMS1RB-TEMP-STATUS-1
050600         MOVE ZERO               TO MMS1RB-TEMP-STATUS-2
050700     ELSE
050800         GO TO Z900-ABORT.
050900*
051000 D999-DA-EXIT.
051100     EXIT.
051200/
051300 R000-RANDOM-ACCESS.
051400*
051500     GO TO R100-RA-INVALID-REQUEST
051600           R100-RA-INVALID-REQUEST
051700           R100-RA-INVALID-REQUEST
051800           R100-RA-INVALID-REQUEST
051900           R100-RA-INVALID-REQUEST
052000           R100-RA-INVALID-REQUEST
052100           R110-RA-OPEN-INPUT
052200           R120-RA-OPEN-OUTPUT
052300           R130-RA-OPEN-I-O
052400           R100-RA-INVALID-REQUEST
052500           R100-RA-INVALID-REQUEST
052600           R100-RA-INVALID-REQUEST
052700           R140-RA-READ-RANDOM
052800           R150-RA-WRITE
052900           R160-RA-REWRITE
053000           R170-RA-DELETE
053100           R180-RA-CLOSE
053200           DEPENDING ON MMS1RB-IO-REQUEST.
053300*
053400 R100-RA-INVALID-REQUEST.
053500*
053600     MOVE MMS1RB-IO-REQUEST      TO SS02WS-ABEND-REQUEST.
053700     MOVE '- BAD I-O REQ - MMS1' TO SS02WS-ABENDMSG8.
053800     GO TO Z950-ABORT.
053900*
054000 R110-RA-OPEN-INPUT.
054100*
054200     OPEN INPUT MMS1-FILE-RDM.
054300*
054400     IF  WS-STATUS IS EQUAL TO '97'
054500         MOVE '97'               TO MMS1RB-OPEN-STATUS
054600         MOVE LOW-VALUES         TO WS-RETURN
054700         MOVE ZEROS              TO WS-STATUS.
054800*
054900     MOVE 'RDM'                  TO WS-SM-ACCESS.
055000     MOVE 'INPUT'                TO WS-SM-OPENED.
055100     MOVE '- OPEN  ERROR - MMS1' TO SS02WS-ABENDMSG8.
055200     GO TO R900-RA-CHECK-STATUS.
055300*
055400 R120-RA-OPEN-OUTPUT.
055500*
055600     OPEN OUTPUT MMS1-FILE-RDM.
055700*
055800     IF  WS-STATUS IS EQUAL TO '97'
055900         MOVE '97'               TO MMS1RB-OPEN-STATUS
056000         MOVE LOW-VALUES         TO WS-RETURN
056100         MOVE ZEROS              TO WS-STATUS.
056200*
056300     MOVE 'RDM'                  TO WS-SM-ACCESS.
056400     MOVE 'OUTPUT'               TO WS-SM-OPENED.
056500     MOVE '- OPEN  ERROR - MMS1' TO SS02WS-ABENDMSG8.
056600     GO TO R900-RA-CHECK-STATUS.
056700*
056800 R130-RA-OPEN-I-O.
056900*
057000     OPEN I-O MMS1-FILE-RDM.
057100*
057200     MOVE 'RDM'                  TO WS-SM-ACCESS.
057300     MOVE 'I-O'                  TO WS-SM-OPENED.
057400     MOVE '- OPEN  ERROR - MMS1' TO SS02WS-ABENDMSG8.
057500     GO TO R900-RA-CHECK-STATUS.
057600*
057700 R140-RA-READ-RANDOM.
057800*
057900     SET WS-MAXIMUM-DATA-LENGTH  TO TRUE.
058000     MOVE MMS1IO-KEY             TO MMS1-RDM-KEY.
058100     READ MMS1-FILE-RDM          INTO MMS1IO-RECORD.
058200*
058300     ADD 1                       TO WS-SC-READS.
058400     MOVE '- READ  ERROR - MMS1' TO SS02WS-ABENDMSG8.
058500     GO TO R900-RA-CHECK-STATUS.
058600*
058700 R150-RA-WRITE.
058800*
058900     MOVE MMS1IO-REC-LENGTH      TO WS-DATA-LENGTH.
059000*
059100     WRITE MMS1-RDM-RECORD       FROM MMS1IO-RECORD.
059200*
059300     ADD 1                       TO WS-SC-WRITES.
059400     MOVE '- WRITE ERROR - MMS1' TO SS02WS-ABENDMSG8.
059500     GO TO R900-RA-CHECK-STATUS.
059600*
059700 R160-RA-REWRITE.
059800*
059900     MOVE MMS1IO-REC-LENGTH      TO WS-DATA-LENGTH.
060000*
060100     REWRITE MMS1-RDM-RECORD     FROM MMS1IO-RECORD.
060200*
060300     ADD 1                       TO WS-SC-REWRITES.
060400     MOVE '- REWRT ERROR - MMS1' TO SS02WS-ABENDMSG8.
060500     GO TO R900-RA-CHECK-STATUS.
060600*
060700 R170-RA-DELETE.
060800*
060900     MOVE MMS1IO-KEY             TO MMS1-RDM-KEY.
061000     DELETE MMS1-FILE-RDM RECORD.
061100*
061200     ADD 1                       TO WS-SC-DELETES.
061300     MOVE '- DELET ERROR - MMS1' TO SS02WS-ABENDMSG8.
061400     GO TO R900-RA-CHECK-STATUS.
061500*
061600 R180-RA-CLOSE.
061700*
061800     MOVE SPACE                  TO WS-ACCESS-METHOD.
061900     CLOSE  MMS1-FILE-RDM.
062000*
062100     PERFORM Z100-DISPLAY-THE-SUMMARY
062200        THRU Z199-DTS-EXIT.
062300*
062400     MOVE '- CLOSE ERROR - MMS1' TO SS02WS-ABENDMSG8.
062500     GO TO R900-RA-CHECK-STATUS.
062600*
062700 R900-RA-CHECK-STATUS.
062800*
062900     MOVE WS-STATUS              TO MMS1RB-STATUS.
063000     MOVE WS-RETURN              TO MMS1RB-RETURN.
063100*
063200     IF  MMS1RB-STATUS IS EQUAL  TO MMS1RB-TEMP-STATUS-1   OR
063300         MMS1RB-STATUS IS EQUAL  TO MMS1RB-TEMP-STATUS-2   OR
063400         MMS1RB-STATUS IS EQUAL  TO MMS1RB-PERM-STATUS-1   OR
063500         MMS1RB-STATUS IS EQUAL  TO MMS1RB-PERM-STATUS-2   OR
063600         MMS1RB-STATUS IS EQUAL  TO MMS1RB-PERM-STATUS-3   OR
063700         MMS1RB-STATUS IS EQUAL  TO MMS1RB-PERM-STATUS-4   OR
063800         MMS1RB-STATUS IS EQUAL  TO MMS1RB-PERM-STATUS-5
063900         MOVE ZERO               TO MMS1RB-TEMP-STATUS-1
064000         MOVE ZERO               TO MMS1RB-TEMP-STATUS-2
064100     ELSE
064200         GO TO Z900-ABORT.
064300*
064400 R999-RA-EXIT.
064500     EXIT.
064600/
064700 S000-SEQUENTIAL-ACCESS.
064800*
064900     GO TO S110-SA-OPEN-INPUT
065000           S120-SA-OPEN-OUTPUT
065100           S130-SA-OPEN-I-O
065200           S100-SA-INVALID-REQUEST
065300           S100-SA-INVALID-REQUEST
065400           S100-SA-INVALID-REQUEST
065500           S100-SA-INVALID-REQUEST
065600           S100-SA-INVALID-REQUEST
065700           S100-SA-INVALID-REQUEST
065800           S140-SA-START-GTE
065900           S150-SA-START-EQUAL
066000           S160-SA-READ-SEQ
066100           S100-SA-INVALID-REQUEST
066200           S170-SA-WRITE
066300           S180-SA-REWRITE
066400           S190-SA-DELETE
066500           S200-SA-CLOSE
066600           DEPENDING ON MMS1RB-IO-REQUEST.
066700*
066800 S100-SA-INVALID-REQUEST.
066900*
067000     MOVE MMS1RB-IO-REQUEST      TO SS02WS-ABEND-REQUEST.
067100     MOVE '- BAD I-O REQ - MMS1' TO SS02WS-ABENDMSG8.
067200     GO TO Z950-ABORT.
067300*
067400 S110-SA-OPEN-INPUT.
067500*
067600     OPEN INPUT MMS1-FILE-SEQ.
067700*
067800     IF  WS-STATUS IS EQUAL TO '97'
067900         MOVE '97'               TO MMS1RB-OPEN-STATUS
068000         MOVE LOW-VALUES         TO WS-RETURN
068100         MOVE ZEROS              TO WS-STATUS.
068200*
068300     MOVE 'SEQ'                  TO WS-SM-ACCESS.
068400     MOVE 'INPUT'                TO WS-SM-OPENED.
068500     MOVE '- OPEN  ERROR - MMS1' TO SS02WS-ABENDMSG8.
068600     GO TO S900-SA-CHECK-STATUS.
068700*
068800 S120-SA-OPEN-OUTPUT.
068900*
069000     OPEN OUTPUT MMS1-FILE-SEQ.
069100*
069200     IF  WS-STATUS IS EQUAL TO '97'
069300         MOVE '97'               TO MMS1RB-OPEN-STATUS
069400         MOVE LOW-VALUES         TO WS-RETURN
069500         MOVE ZEROS              TO WS-STATUS.
069600*
069700     MOVE 'SEQ'                  TO WS-SM-ACCESS.
069800     MOVE 'OUTPUT'               TO WS-SM-OPENED.
069900     MOVE '- OPEN  ERROR - MMS1' TO SS02WS-ABENDMSG8.
070000     GO TO S900-SA-CHECK-STATUS.
070100*
070200 S130-SA-OPEN-I-O.
070300*
070400     OPEN I-O MMS1-FILE-SEQ.
070500*
070600     MOVE 'SEQ'                  TO WS-SM-ACCESS.
070700     MOVE 'I-O'                  TO WS-SM-OPENED.
070800     MOVE '- OPEN  ERROR - MMS1' TO SS02WS-ABENDMSG8.
070900     GO TO S900-SA-CHECK-STATUS.
071000*
071100 S140-SA-START-GTE.
071200*
071300     MOVE MMS1IO-KEY             TO MMS1-SEQ-KEY.
071400     START MMS1-FILE-SEQ
071500           KEY IS NOT LESS THAN MMS1-SEQ-KEY.
071600*
071700     MOVE '- START ERROR - MMS1' TO SS02WS-ABENDMSG8.
071800     GO TO S900-SA-CHECK-STATUS.
071900*
072000 S150-SA-START-EQUAL.
072100*
072200     MOVE MMS1IO-KEY             TO MMS1-SEQ-KEY.
072300     START MMS1-FILE-SEQ
072400           KEY IS EQUAL TO MMS1-SEQ-KEY.
072500*
072600     MOVE '- START ERROR - MMS1' TO SS02WS-ABENDMSG8.
072700     GO TO S900-SA-CHECK-STATUS.
072800*
072900 S160-SA-READ-SEQ.
073000*
073100     SET WS-MAXIMUM-DATA-LENGTH  TO TRUE.
073200     READ MMS1-FILE-SEQ          INTO MMS1IO-RECORD.
073300*
073400     ADD 1                       TO WS-SC-READS.
073500     MOVE '- READ  ERROR - MMS1' TO SS02WS-ABENDMSG8.
073600     GO TO S900-SA-CHECK-STATUS.
073700*
073800 S170-SA-WRITE.
073900*
074000     MOVE MMS1IO-REC-LENGTH      TO WS-DATA-LENGTH.
074100*
074200     WRITE MMS1-SEQ-RECORD       FROM MMS1IO-RECORD.
074300*
074400     ADD 1                       TO WS-SC-WRITES.
074500     MOVE '- WRITE ERROR - MMS1' TO SS02WS-ABENDMSG8.
074600     GO TO S900-SA-CHECK-STATUS.
074700*
074800 S180-SA-REWRITE.
074900*
075000     MOVE MMS1IO-REC-LENGTH      TO WS-DATA-LENGTH.
075100*
075200     REWRITE MMS1-SEQ-RECORD     FROM MMS1IO-RECORD.
075300*
075400     ADD 1                       TO WS-SC-REWRITES.
075500     MOVE '- REWRT ERROR - MMS1' TO SS02WS-ABENDMSG8.
075600     GO TO S900-SA-CHECK-STATUS.
075700*
075800 S190-SA-DELETE.
075900*
076000     MOVE MMS1IO-KEY             TO MMS1-SEQ-KEY.
076100     DELETE MMS1-FILE-SEQ RECORD.
076200*
076300     ADD 1                       TO WS-SC-DELETES.
076400     MOVE '- DELET ERROR - MMS1' TO SS02WS-ABENDMSG8.
076500     GO TO S900-SA-CHECK-STATUS.
076600*
076700 S200-SA-CLOSE.
076800*
076900     MOVE SPACE                  TO WS-ACCESS-METHOD.
077000     CLOSE MMS1-FILE-SEQ.
077100*
077200     PERFORM Z100-DISPLAY-THE-SUMMARY
077300        THRU Z199-DTS-EXIT.
077400*
077500     MOVE '- CLOSE ERROR - MMS1' TO SS02WS-ABENDMSG8.
077600     GO TO S900-SA-CHECK-STATUS.
077700*
077800 S900-SA-CHECK-STATUS.
077900*
078000     MOVE WS-STATUS              TO MMS1RB-STATUS.
078100     MOVE WS-RETURN              TO MMS1RB-RETURN.
078200*
078300     IF  MMS1RB-STATUS IS EQUAL  TO MMS1RB-TEMP-STATUS-1   OR
078400         MMS1RB-STATUS IS EQUAL  TO MMS1RB-TEMP-STATUS-2   OR
078500         MMS1RB-STATUS IS EQUAL  TO MMS1RB-PERM-STATUS-1   OR
078600         MMS1RB-STATUS IS EQUAL  TO MMS1RB-PERM-STATUS-2   OR
078700         MMS1RB-STATUS IS EQUAL  TO MMS1RB-PERM-STATUS-3   OR
078800         MMS1RB-STATUS IS EQUAL  TO MMS1RB-PERM-STATUS-4   OR
078900         MMS1RB-STATUS IS EQUAL  TO MMS1RB-PERM-STATUS-5
079000         MOVE ZERO               TO MMS1RB-TEMP-STATUS-1
079100         MOVE ZERO               TO MMS1RB-TEMP-STATUS-2
079200     ELSE
079300         GO TO Z900-ABORT.
079400*
079500 S999-SA-EXIT.
079600     EXIT.
079700/
079800 Z000-PROGRAM-INITIALIZATION.
079900*
080000************************************************************
080100*    THE CALLING PROGRAM MUST SUPPLY THE PROGRAM ID TO     *
080200*    ENSURE THAT ANY SUBSEQUENT ABEND CAN BE TRACED TO     *
080300*    IT.  IF THE PROGRAM ID IS NOT SUPPLIED, THE ACCESS    *
080400*    ROUTINE ABENDS HERE.                                  *
080500************************************************************
080600*
080700     IF  SS02WS-ABEND-PROG-ID IS EQUAL TO SPACES
080800         MOVE '- BAD PROG NAME - MMS1'
080900                                 TO SS02WS-ABENDMSG8
081000         MOVE 'BADPRO'           TO SS02WS-ABEND-PROG-ID
081100         GO TO Z950-ABORT.
081200*
081300     MOVE SS02WS-ABEND-PROG-ID   TO WS-SM-PROG-ID.
081400*
081500*************************************************************
081600*    SINCE THIS ROUTINE IS PERFORMED ONLY ONCE DURING       *
081700*    THE EXECUTION OF A CALLING PROGRAM, THE FIRST          *
081800*    REQUEST MUST BE FOR AN OPEN.  SET THE ACCESS METHOD    *
081900*    (SEQENTIAL, DYNAMIC, OR RANDOM), OR ISSUE AN ABEND     *
082000*    IF THE REQUEST IS NOT FOR AN OPEN.                     *
082100*************************************************************
082200*
082300     IF  MMS1RB-OPEN-INPUT    OR
082400         MMS1RB-OPEN-OUTPUT   OR
082500         MMS1RB-OPEN-IO
082600         MOVE 'S'                TO WS-ACCESS-METHOD.
082700*
082800     IF  MMS1RB-OPEN-INPUT-DYN    OR
082900         MMS1RB-OPEN-OUTPUT-DYN   OR
083000         MMS1RB-OPEN-IO-DYN
083100         MOVE 'D'                TO WS-ACCESS-METHOD.
083200*
083300     IF  MMS1RB-OPEN-INPUT-RDM    OR
083400         MMS1RB-OPEN-OUTPUT-RDM   OR
083500         MMS1RB-OPEN-IO-RDM
083600         MOVE 'R'                TO WS-ACCESS-METHOD.
083700*
083800     IF  ACCESS-METHOD-NOT-SET
083900         MOVE 'SMMS1'            TO SS02WS-ABEND-CODE-X
084000         MOVE MMS1RB-IO-REQUEST  TO WS-IO-REQUEST
084100         MOVE WS-IO-REQUEST      TO SS02WS-ABEND-REQUEST
084200         MOVE 'ILLEGAL ACCESS REQUEST'
084300                                 TO SS02WS-ABENDMSG8
084400         GO TO Z950-ABORT.
084500*
084600 Z099-PI-EXIT.
084700     EXIT.
084800*
084900 Z100-DISPLAY-THE-SUMMARY.
085000*
085100     MOVE WS-SC-READS            TO WS-SM-READS.
085200     MOVE WS-SC-WRITES           TO WS-SM-WRITES.
085300     MOVE WS-SC-REWRITES         TO WS-SM-REWRITES.
085400     MOVE WS-SC-DELETES          TO WS-SM-DELETES.
085500     DISPLAY WS-SUMMARY-MESSAGE.
085600*
085700     INITIALIZE WS-SUMMARY-COUNTERS.
085800*
085900 Z199-DTS-EXIT.
086000     EXIT.
086100/
086200 Z900-ABORT.
086300*
086400     MOVE 'SMMS1'                TO SS02WS-ABEND-CODE-X.
086500     MOVE MMS1RB-STATUS          TO SS02WS-ABEND-STATUS.
086600     MOVE MMS1RB-RETURN-CODE     TO SS02WS-ABEND-RETURN.
086700     MOVE MMS1RB-FUNCTION-CODE   TO SS02WS-ABEND-FUNCTION.
086800     MOVE MMS1RB-FEEDBACK-CODE   TO SS02WS-ABEND-FEEDBACK.
086900     MOVE MMS1RB-IO-REQUEST      TO WS-IO-REQUEST.
087000     MOVE WS-IO-REQUEST          TO SS02WS-ABEND-REQUEST.
087100*
087200 Z950-ABORT.
087300*
087400     PERFORM SS02PD-ABEND        THRU SS02PD-ABEND-EXIT.
087500*
087600 Z999-ABORT-EXIT.
087700     EXIT.
087800/
087900********************************
088000*    STANDARD ABEND ROUTINE    *
088100********************************
088200*
088300     COPY SS02PD.
088400*
