000100 IDENTIFICATION DIVISION.
000200 PROGRAM-ID. MBTFAB.
000201*       RELEASE DATE=08/24/01.        REVISION LEVEL=08.06      * ++++++++
000202***************************************************************** ++++++++
000203*WARNING.  THIS COMPUTER PROGRAM IS PROTECTED BY COPYRIGHT LAW  * ++++++++
000204*AND INTERNATIONAL TREATIES.  UNAUTHORIZED REPRODUCTION OR      * ++++++++
000205*DISTRIBUTION OF THIS PROGRAM, OR ANY PORTION OF IT, MAY RESULT * ++++++++
000206*IN SEVERE CIVIL AND CRIMINAL PENATLIES, AND WILL BE PROSECUTED * ++++++++
000207*TO THE MAXIMUM EXTENT POSSIBLE UNDER THE LAW.                  * ++++++++
000208*COPYRIGHT 1999 PAYSYS INTERNATIONAL, INC.  ALL RIGHTS RESERVED.* ++++++++
000209***************************************************************** ++++++++
000300******************************************************************
000400*                                                                *
000500*    ---------------MERCHANT BANKCARD SYSTEM---------------      *
000600*                                                                *
000700*          MBTFAB - MBTF FILE COMMON BATCH I/O ROUTINE           *
000800*                                                                *
000900*                VARIABLE LENGTH SEQUENTIAL FILE                 *
001000*                                                                *
001100*    --------------TRANSACTION BACKUP FILE-----------------      *
001200*                                                                *
001300******************************************************************
001400*                                                                *
001500*    COPYBOOKS WHICH MUST APPEAR IN YOUR BATCH PROGRAM:          *
001600*                                                                *
001700*    (WS) MB00RB - BATCH REQUEST BLOCK                           *
001800*    (WS) MBTFIO - I/O AREA                                      *
001900*    (WS) SS02WS - STANDARD ABEND ROUTINE WORK AREA              *
002000*    (PD) MB00PD - PROCEDURE DIVISION CALL STATEMENT             *
002100*                                                                *
002200******************************************************************
002300*                                                                *
002400*    TO USE THIS ROUTINE.........                                *
002500*                                                                *
002600* 1. HOUSEKEEPING DUTIES:                                        *
002700*                                                                *
002800*    A. IN THE HOUSEKEEPING SECTION OF YOUR PROGRAM, MOVE THE    *
002900*       NAME OF THE PROGRAM USING THIS ROUTINE TO THE FIELD      *
003000*       'SS02WS-ABEND-PROG-ID'. THIS FIELD IS USED AS PART OF THE*
003100*       STANDARD ABEND MESSAGES IF A PROGRAM ABORTS WITHIN THIS  *
003200*       ACCESS ROUTINE.  IF THE FIELD 'SS02WS-ABEND-PROG-ID' IS  *
003300*       LEFT BLANK, THIS ROUTINE WILL FORCE AN ABEND SITUATION.  *
003400*                                                                *
003500*    B. BEFORE PERFORMING THE ACCESS ROUTINE, MOVE A LITERAL TO  *
003600*       THE FIELD 'SS02WS-ABEND-LOCATION'. THIS FIELD CAN BE USED*
003700*       TO INDICATE WHERE THE ROUTINE IS BEING 'FROM' IN CASE    *
003800*       OF ABNORMAL TERMINATION. (E.G. DEBUGGING ASSISTANCE)     *
003900*                                                                *
004000*    C. SET THE RETURN CODES WHICH YOU WANT, FOR THE DURATION    *
004100*       OF THIS PROGRAM, TO BE CONSIDERED VALID IN THE FIELDS    *
004200*       'MBTFRB-PERM-STATUS-1 TO -5'. THESE FIELDS ARE INITIAL-  *
004300*       IZED TO ZERO (AS ZERO IS ALWAYS AN ACCEPTABLE RETURN     *
004400*       CODE).  THIS ROUTINE WILL (IN ALL I-O REQUESTS) SUCCESS- *
004500*       FULLY SATISFY YOUR PERFORM REQUEST IF ANY OF THE REQUESTS*
004600*       ENCOUNTER ANY OF THE PERMANENT STATUS CODES 1-5.         *
004700*                                                                *
004800* 2. SET ONE OF THE FOLLOWING CONDITION NAMES TO 'TRUE' TO SET   *
004900*    THE FILE REQUEST:                                           *
005000*                                                                *
005100*        CONDITION NAME                EXPLANATION               *
005200*   *----------------------***---------------------------------* *
005300*                                                                *
005400*    MBTFRB-OPEN-INPUT      - OPEN THE FILE AS INPUT.            *
005500*    MBTFRB-OPEN-OUTPUT     - OPEN THE FILE AS OUTPUT.           *
005600*    MBTFRB-OPEN-IO         - OPEN THE FILE AS I-O.              *
005700*    MBTFRB-READ-SEQ        - READ THE NEXT SEQUENTIAL RECORD    *
005800*                             IN THE FILE.                       *
005900*    MBTFRB-WRITE           - WRITE A RECORD (THE RECORD MUST    *
006000*                             BE IN 'MBTFIO-RECORD').            *
006100*    MBTFRB-REWRITE         - REWRITE A RECORD (THE RECORD MUST  *
006200*                             BE IN 'MBTFIO-RECORD').            *
006300*    MBTFRB-CLOSE           - CLOSE THE FILE.                    *
006400*                                                                *
006500* 3. SET ONE OR BOTH OF THE 'TEMPORARY' RETURN CODE FIELDS       *
006600*    'MBTFRB-TEMP-STATUS-1 AND -2'.  THESE TWO RETURN CODES      *
006700*    ARE TO BE USED ON A PER USE BASIS OF THIS ROUTINE.          *
006800*    THESE TWO RETURN CODES ARE INITIALIZED TO ZERO AND ARE      *
006900*    BOTH RESET TO ZERO AFTER EACH CALL OF THIS PROGRAM.         *
007000*    THE TEMPORARY RETURN CODE FIELDS ARE CHECKED FIRST AND      *
007100*    THEN THE PERMANENT RETURN CODES (POINT 1.C ABOVE) WILL      *
007200*    BE CHECKED.                                                 *
007300*                                                                *
007400* 4. PERFORM MB00PD-MBTF-ACCESS THRU MB00PD-MBTF-ACCESS-EXIT.    *
007500*                                                                *
007600*    A. YOU MUST HANDLE MANUALLY ANY RETURN CODE (EITHER TEMP    *
007700*       OR PERM) THAT YOU REQUESTED, AND IF NEEDED, YOUR OWN     *
007800*       ABEND REQUEST.                                           *
007900*                                                                *
008000*    B. ALL ACCESS ROUTINES MOVE THE FILE STATUS TO THE FIELD    *
008100*       SS02WS-FILE-STATUS IN COPYBOOK SS02WS. USE THE '88' LEVEL*
008200*       NAMES TO TEST YOUR MANUAL STATUS CHECKS.  ALSO IN COPY   *
008300*       BOOK SS02WS, USE THE 'WS-' FILE STATUS DATA NAMES WHEN   *
008400*       SETTING THE PERM OR TEMP FILE STATUS CODES PRIOR TO      *
008500*       PERFORMING THE ACCESS ROUTINE.                           *
008600*                                                                *
008700******************************************************************
008800/
008900 ENVIRONMENT DIVISION.
009000*
009100 CONFIGURATION SECTION.
009200 SOURCE-COMPUTER.            IBM-4341.
009300 OBJECT-COMPUTER.            IBM-4341.
009400*
009500 INPUT-OUTPUT SECTION.
009600*
009700 FILE-CONTROL.
009800*
009900     SELECT  MBTF-FILE
010000             ASSIGN              TO SYS002-UT-3380-S-MBTF
010100             FILE STATUS         IS WS-STATUS.
010200/
010300 DATA DIVISION.
010400*
010500 FILE SECTION.
010600*
010700 FD  MBTF-FILE
010800     RECORDING MODE IS V
010900     RECORD CONTAINS 07 TO 16380 CHARACTERS
011000     BLOCK CONTAINS 0 CHARACTERS.
011100*
011200 01  MBTF-RECORD.
011300     03  MBTF-FILE-CODE      PIC  X(02).
011400         88  MBTF-FILE-HDR                   VALUE 'TF'.
011500         88  MBTF-FILE-IS-MML1               VALUE 'L1'.
011600         88  MBTF-FILE-IS-MMT1               VALUE 'T1'.
011700*
011800     03  MBTF-VAR-LEN        PIC S9(08)      BINARY.
011900*
012000     03  MBTF-VAR-DATA.
012100         05  MBTF-DATA       PIC  X(01)      OCCURS
012200                                             1 TO 16374 TIMES
012300                                             DEPENDING ON
012400                                             WS-DATA-LENGTH.
012500*
012600/
012700 WORKING-STORAGE SECTION.
012800*
012900 01  FILLER                  PIC  X(12)      VALUE '** WS-BEG **'.
013000*
013100 01  FILLER                  PIC  X(18)      VALUE
013200     'FIRST CALL SWITCH:'.
013300*
013400 01  WS-FIRST-CALL-SW        PIC  X(01)      VALUE 'Y'.
013500     88  WS-FIRST-CALL-SW-IS-YES             VALUE 'Y'.
013600*
013700 01  FILLER                  PIC  X(12)      VALUE
013800     'FILE STATUS:'.
013900*
014000 01  WS-STATUS               PIC  X(02)      VALUE ZERO.
014100*
014200 01  FILLER                  PIC  X(12)      VALUE
014300     'I-O REQUEST:'.
014400*
014500 01  WS-IO-REQUEST           PIC  9(02)      VALUE ZERO.
014600*
014700 01  FILLER                  PIC  X(12)      VALUE
014800     'DATA LENGTH:'.
014900*
015000 01  WS-DATA-LENGTH          PIC S9(08)      BINARY
015100                                             VALUE ZERO.
015200     88  WS-MAXIMUM-DATA-LENGTH              VALUE +16374.
015300*
015400 01  FILLER                  PIC  X(28)      VALUE
015500     'PROCESSING SUMMARY COUNTERS:'.
015600*
015700 01  WS-SUMMARY-COUNTERS.
015800     03  WS-SC-READS         PIC S9(09)      PACKED-DECIMAL
015900                                             VALUE ZERO.
016000     03  WS-SC-WRITES        PIC S9(09)      PACKED-DECIMAL
016100                                             VALUE ZERO.
016200     03  WS-SC-REWRITES      PIC S9(09)      PACKED-DECIMAL
016300                                             VALUE ZERO.
016400*
016500 01  FILLER                  PIC  X(27)      VALUE
016600     'PROCESSING SUMMARY MESSAGE:'.
016700*
016800 01  WS-SUMMARY-MESSAGE.
016900     03  FILLER              PIC  X(10)      VALUE '(PGMSTAT) '.
017000     03  WS-SM-PROG-ID       PIC  X(06)      VALUE SPACES.
017100     03  FILLER              PIC  X(03)      VALUE ' - '.
017200     03  WS-SM-FILE-ID       PIC  X(04)      VALUE 'MBTF'.
017300     03  FILLER              PIC  X(11)      VALUE ' - OPENED: '.
017400     03  WS-SM-ACCESS        PIC  X(03)      VALUE SPACES.
017500     03  FILLER              PIC  X(01)      VALUE SPACE.
017600     03  WS-SM-OPENED        PIC  X(06)      VALUE SPACES.
017700     03  FILLER              PIC  X(01)      VALUE SPACE.
017800     03  WS-SM-READS         PIC  ZZZZZZZZ9.
017900     03  FILLER              PIC  X(08)      VALUE ' READS, '.
018000     03  WS-SM-WRITES        PIC  ZZZZZZZZ9.
018100     03  FILLER              PIC  X(09)      VALUE ' WRITES, '.
018200     03  WS-SM-REWRITES      PIC  ZZZZZZZZ9.
018300     03  FILLER              PIC  X(09)      VALUE ' REWRITES'.
018400/
018500     COPY SS00WS.
018600*
018700 01  FILLER                  PIC  X(12)      VALUE '** WS-END **'.
018800/
018900 LINKAGE SECTION.
019000*
019100****************************
019200*    FILE REQUEST BLOCK    *
019300****************************
019400*
019500     COPY MB00RB REPLACING ==:MB00:== BY ==MBTF==.
019600*
019700******************
019800*    I/O AREA    *
019900******************
020000*
020100     COPY MBTFIO.
020200*
020300*********************************
020400*    ABEND ROUTINE WORK AREA    *
020500*********************************
020600*
020700     COPY SS02WS.
020800/
020900 PROCEDURE DIVISION          USING MBTFRB-WORK-AREA,
021000                                   MBTFIO-RECORD,
021100                                   SS02WS-ABEND-WORK-AREA.
021200*
021300 A000-MAIN-LINE-SECTION.
021400*
021500*
021600******************************************
021700*    SET THE ABEND WORK AREA IF THIS     *
021800*    IS THE FIRST CALL OF THE PROGRAM    *
021900******************************************
022000*
022100     IF  WS-FIRST-CALL-SW-IS-YES
022200         PERFORM Z000-PROGRAM-INITIALIZATION
022300            THRU Z099-PI-EXIT.
022400*
022500**************************
022600*    PROCESS THE FILE    *
022700**************************
022800*
022900     PERFORM S000-SEQUENTIAL-ACCESS
023000        THRU S999-SA-EXIT.
023100*
023200************************************************
023300*    MOVE THE STATUS CODE TO ITS RESPECTIVE    *
023400*    FIELD IN COPYBOOK SS02WS.                 *
023500************************************************
023600*
023700     MOVE MBTFRB-STATUS          TO SS02WS-FILE-STATUS.
023800*
023900 A999-GOBACK.
024000*
024100     GOBACK.
024200/
024300 S000-SEQUENTIAL-ACCESS.
024400*
024500     GO TO S110-SA-OPEN-INPUT
024600           S120-SA-OPEN-OUTPUT
024700           S130-SA-OPEN-I-O
024800           S100-SA-INVALID-REQUEST
024900           S100-SA-INVALID-REQUEST
025000           S100-SA-INVALID-REQUEST
025100           S100-SA-INVALID-REQUEST
025200           S100-SA-INVALID-REQUEST
025300           S100-SA-INVALID-REQUEST
025400           S100-SA-INVALID-REQUEST
025500           S100-SA-INVALID-REQUEST
025600           S140-SA-READ-SEQ
025700           S100-SA-INVALID-REQUEST
025800           S150-SA-WRITE
025900           S160-SA-REWRITE
026000           S100-SA-INVALID-REQUEST
026100           S170-SA-CLOSE
026200           DEPENDING ON MBTFRB-IO-REQUEST.
026300*
026400 S100-SA-INVALID-REQUEST.
026500*
026600     MOVE MBTFRB-IO-REQUEST      TO SS02WS-ABEND-REQUEST.
026700     MOVE '- BAD I-O REQ - MBTF' TO SS02WS-ABENDMSG8.
026800     GO TO Z950-ABORT.
026900*
027000 S110-SA-OPEN-INPUT.
027100*
027200     OPEN INPUT MBTF-FILE.
027300*
027400     MOVE 'SEQ'                  TO WS-SM-ACCESS.
027500     MOVE 'INPUT'                TO WS-SM-OPENED.
027600     MOVE '- OPEN  ERROR - MBTF' TO SS02WS-ABENDMSG8.
027700     GO TO S900-SA-CHECK-STATUS.
027800*
027900 S120-SA-OPEN-OUTPUT.
028000*
028100     OPEN OUTPUT MBTF-FILE.
028200*
028300     MOVE 'SEQ'                  TO WS-SM-ACCESS.
028400     MOVE 'OUTPUT'               TO WS-SM-OPENED.
028500     MOVE '- OPEN  ERROR - MBTF' TO SS02WS-ABENDMSG8.
028600     GO TO S900-SA-CHECK-STATUS.
028700*
028800 S130-SA-OPEN-I-O.
028900*
029000     OPEN I-O MBTF-FILE.
029100*
029200     MOVE 'SEQ'                  TO WS-SM-ACCESS.
029300     MOVE 'I-O'                  TO WS-SM-OPENED.
029400     MOVE '- OPEN  ERROR - MBTF' TO SS02WS-ABENDMSG8.
029500     GO TO S900-SA-CHECK-STATUS.
029600*
029700 S140-SA-READ-SEQ.
029800*
029900     SET WS-MAXIMUM-DATA-LENGTH  TO TRUE.
030000     READ MBTF-FILE              INTO MBTFIO-RECORD.
030100*
030200     ADD 1                       TO WS-SC-READS.
030300     MOVE '- READ  ERROR - MBTF' TO SS02WS-ABENDMSG8.
030400     GO TO S900-SA-CHECK-STATUS.
030500*
030600 S150-SA-WRITE.
030700*
030800     MOVE MBTFIO-VAR-LEN         TO WS-DATA-LENGTH.
030900*
031000     WRITE MBTF-RECORD           FROM MBTFIO-RECORD.
031100*
031200     ADD 1                       TO WS-SC-WRITES.
031300     MOVE '- WRITE ERROR - MBTF' TO SS02WS-ABENDMSG8.
031400     GO TO S900-SA-CHECK-STATUS.
031500*
031600 S160-SA-REWRITE.
031700*
031800     MOVE MBTFIO-VAR-LEN         TO WS-DATA-LENGTH.
031900*
032000     REWRITE MBTF-RECORD         FROM MBTFIO-RECORD.
032100*
032200     ADD 1                       TO WS-SC-REWRITES.
032300     MOVE '- REWRT ERROR - MBTF' TO SS02WS-ABENDMSG8.
032400     GO TO S900-SA-CHECK-STATUS.
032500*
032600 S170-SA-CLOSE.
032700*
032800     MOVE SPACE                  TO WS-FIRST-CALL-SW.
032900     CLOSE MBTF-FILE.
033000*
033100     PERFORM Z100-DISPLAY-THE-SUMMARY
033200        THRU Z199-DTS-EXIT.
033300*
033400     MOVE '- CLOSE ERROR - MBTF' TO SS02WS-ABENDMSG8.
033500     GO TO S900-SA-CHECK-STATUS.
033600*
033700 S900-SA-CHECK-STATUS.
033800*
033900     MOVE WS-STATUS              TO MBTFRB-STATUS.
034000*
034100     IF  MBTFRB-STATUS IS EQUAL  TO MBTFRB-TEMP-STATUS-1   OR
034200         MBTFRB-STATUS IS EQUAL  TO MBTFRB-TEMP-STATUS-2   OR
034300         MBTFRB-STATUS IS EQUAL  TO MBTFRB-PERM-STATUS-1   OR
034400         MBTFRB-STATUS IS EQUAL  TO MBTFRB-PERM-STATUS-2   OR
034500         MBTFRB-STATUS IS EQUAL  TO MBTFRB-PERM-STATUS-3   OR
034600         MBTFRB-STATUS IS EQUAL  TO MBTFRB-PERM-STATUS-4   OR
034700         MBTFRB-STATUS IS EQUAL  TO MBTFRB-PERM-STATUS-5
034800         MOVE ZERO               TO MBTFRB-TEMP-STATUS-1
034900         MOVE ZERO               TO MBTFRB-TEMP-STATUS-2
035000     ELSE
035100         GO TO Z900-ABORT.
035200*
035300 S999-SA-EXIT.
035400     EXIT.
035500/
035600 Z000-PROGRAM-INITIALIZATION.
035700*
035800************************************************************
035900*    THE CALLING PROGRAM MUST SUPPLY THE PROGRAM ID TO     *
036000*    ENSURE THAT ANY SUBSEQUENT ABEND CAN BE TRACED TO     *
036100*    IT.  IF THE PROGRAM ID IS NOT SUPPLIED, THE ACCESS    *
036200*    ROUTINE ABENDS HERE.                                  *
036300************************************************************
036400*
036500     IF  SS02WS-ABEND-PROG-ID IS EQUAL TO SPACES
036600         MOVE '- BAD PROG NAME - MBTF'
036700                                 TO SS02WS-ABENDMSG8
036800         MOVE 'BADPRO'           TO SS02WS-ABEND-PROG-ID
036900         GO TO Z950-ABORT.
037000*
037100     MOVE SS02WS-ABEND-PROG-ID   TO WS-SM-PROG-ID.
037200*
037300*************************************************************
037400*    SINCE THIS ROUTINE IS PERFORMED ONLY ONCE DURING       *
037500*    THE EXECUTION OF A CALLING PROGRAM, THE FIRST          *
037600*    REQUEST MUST BE FOR AN OPEN.  SET THE ACCESS METHOD    *
037700*    (SEQENTIAL, DYNAMIC, OR RANDOM), OR ISSUE AN ABEND     *
037800*    IF THE REQUEST IS NOT FOR AN OPEN.                     *
037900*************************************************************
038000*
038100     IF  MBTFRB-OPEN-INPUT    OR
038200         MBTFRB-OPEN-OUTPUT   OR
038300         MBTFRB-OPEN-IO
038400         MOVE 'N'                TO WS-FIRST-CALL-SW
038500     ELSE
038600         MOVE 'ILLEGAL ACCESS REQUEST'
038700                                 TO SS02WS-ABENDMSG8
038800         GO TO Z950-ABORT.
038900*
039000 Z099-PI-EXIT.
039100     EXIT.
039200*
039300 Z100-DISPLAY-THE-SUMMARY.
039400*
039500     MOVE WS-SC-READS            TO WS-SM-READS.
039600     MOVE WS-SC-WRITES           TO WS-SM-WRITES.
039700     MOVE WS-SC-REWRITES         TO WS-SM-REWRITES.
039800     DISPLAY WS-SUMMARY-MESSAGE.
039900*
040000     INITIALIZE WS-SUMMARY-COUNTERS.
040100*
040200 Z199-DTS-EXIT.
040300     EXIT.
040400/
040500 Z900-ABORT.
040600*
040700     MOVE 'SMBTF'                TO SS02WS-ABEND-CODE-X.
040800     MOVE MBTFRB-STATUS          TO SS02WS-ABEND-STATUS.
040900     MOVE MBTFRB-IO-REQUEST      TO WS-IO-REQUEST.
041000     MOVE WS-IO-REQUEST          TO SS02WS-ABEND-REQUEST.
041100*
041200 Z950-ABORT.
041300*
041400     PERFORM SS02PD-ABEND        THRU SS02PD-ABEND-EXIT.
041500*
041600 Z999-ABORT-EXIT.
041700     EXIT.
041800/
041900********************************
042000*    STANDARD ABEND ROUTINE    *
042100********************************
042200*
042300     COPY SS02PD.
042400*
