000100 IDENTIFICATION DIVISION.
000200 PROGRAM-ID. MMEXAB.
000201*       RELEASE DATE=08/24/01.        REVISION LEVEL=08.06      * ++++++++
000202***************************************************************** ++++++++
000203*WARNING.  THIS COMPUTER PROGRAM IS PROTECTED BY COPYRIGHT LAW  * ++++++++
000204*AND INTERNATIONAL TREATIES.  UNAUTHORIZED REPRODUCTION OR      * ++++++++
000205*DISTRIBUTION OF THIS PROGRAM, OR ANY PORTION OF IT, MAY RESULT * ++++++++
000206*IN SEVERE CIVIL AND CRIMINAL PENATLIES, AND WILL BE PROSECUTED * ++++++++
000207*TO THE MAXIMUM EXTENT POSSIBLE UNDER THE LAW.                  * ++++++++
000208*COPYRIGHT 1999 PAYSYS INTERNATIONAL, INC.  ALL RIGHTS RESERVED.* ++++++++
000209***************************************************************** ++++++++
000300******************************************************************
000400*                                                                *
000500*    ---------------MERCHANT BANKCARD SYSTEM---------------      *
000600*                                                                *
000700*          MMEXAB - MMEX FILE COMMON BATCH I/O ROUTINE           *
000800*                                                                *
000900*                    FIXED LENGTH VSAM FILE                      *
001000*                                                                *
001100*    ----------EXTRACT AND MAINTENANCE FILE ACCESS---------      *
001200*                                                                *
001300******************************************************************
001400*                                                                *
001500*    COPYBOOKS WHICH MUST APPEAR IN YOUR BATCH PROGRAM:          *
001600*                                                                *
001700*    (WS) MB00RB - BATCH REQUEST BLOCK                           *
001800*    (WS) MMEXIO - I/O AREA                                      *
001900*    (WS) SS02WS - STANDARD ABEND ROUTINE WORK AREA              *
002000*    (PD) MB00PD - PROCEDURE DIVISION CALL STATEMENT             *
002100*                                                                *
002200******************************************************************
002300*                                                                *
002400*    TO USE THIS ROUTINE.........                                *
002500*                                                                *
002600* 1. HOUSEKEEPING DUTIES:                                        *
002700*                                                                *
002800*    A. IN THE HOUSEKEEPING SECTION OF YOUR PROGRAM, MOVE THE    *
002900*       NAME OF THE PROGRAM USING THIS ROUTINE TO THE FIELD      *
003000*       'SS02WS-ABEND-PROG-ID'. THIS FIELD IS USED AS PART OF THE*
003100*       STANDARD ABEND MESSAGES IF A PROGRAM ABORTS WITHIN THIS  *
003200*       ACCESS ROUTINE.  IF THE FIELD 'SS02WS-ABEND-PROG-ID' IS  *
003300*       LEFT BLANK, THIS ROUTINE WILL FORCE AN ABEND SITUATION.  *
003400*                                                                *
003500*    B. BEFORE PERFORMING THE ACCESS ROUTINE, MOVE A LITERAL TO  *
003600*       THE FIELD 'SS02WS-ABEND-LOCATION'. THIS FIELD CAN BE USED*
003700*       TO INDICATE WHERE THE ROUTINE IS BEING 'FROM' IN CASE    *
003800*       OF ABNORMAL TERMINATION. (E.G. DEBUGGING ASSISTANCE)     *
003900*                                                                *
004000*    C. SET THE RETURN CODES WHICH YOU WANT, FOR THE DURATION    *
004100*       OF THIS PROGRAM, TO BE CONSIDERED VALID IN THE FIELDS    *
004200*       'MMEXRB-PERM-STATUS-1 TO -5'. THESE FIELDS ARE INITIAL-  *
004300*       IZED TO ZERO (AS ZERO IS ALWAYS AN ACCEPTABLE RETURN     *
004400*       CODE).  THIS ROUTINE WILL (IN ALL I-O REQUESTS) SUCCESS- *
004500*       FULLY SATISFY YOUR PERFORM REQUEST IF ANY OF THE REQUESTS*
004600*       ENCOUNTER ANY OF THE PERMANENT STATUS CODES 1-5.         *
004700*                                                                *
004800* 2. SET ONE OF THE FOLLOWING CONDITION NAMES TO 'TRUE' TO SET   *
004900*    THE FILE REQUEST:                                           *
005000*                                                                *
005100*        CONDITION NAME                EXPLANATION               *
005200*   *----------------------***---------------------------------* *
005300*                                                                *
005400*    MMEXRB-OPEN-INPUT      - OPEN THE FILE AS INPUT WITH        *
005500*                             SEQUENTIAL ACCESS.                 *
005600*    MMEXRB-OPEN-OUTPUT     - OPEN THE FILE AS OUTPUT WITH       *
005700*                             SEQUENTIAL ACCESS.                 *
005800*    MMEXRB-OPEN-IO         - OPEN THE FILE AS I-O WITH          *
005900*                             SEQUENTIAL ACCESS.                 *
006000*    MMEXRB-OPEN-INPUT-DYN  - OPEN THE FILE AS INPUT WITH        *
006100*                             DYNAMIC ACCESS.                    *
006200*    MMEXRB-OPEN-OUTPUT-DYN - OPEN THE FILE AS OUTPUT WITH       *
006300*                             DYNAMIC ACCESS.                    *
006400*    MMEXRB-OPEN-IO-DYN     - OPEN THE FILE AS I-O WITH          *
006500*                             DYNAMIC ACCESS.                    *
006600*    MMEXRB-OPEN-INPUT-RDM  - OPEN THE FILE AS INPUT WITH        *
006700*                             RANDOM ACCESS.                     *
006800*    MMEXRB-OPEN-OUTPUT-RDM - OPEN THE FILE AS OUTPUT WITH       *
006900*                             RANDOM ACCESS.                     *
007000*    MMEXRB-OPEN-IO-RDM     - OPEN THE FILE AS I-O WITH          *
007100*                             RANDOM ACCESS.                     *
007200*    MMEXRB-START-GTE       - START THE FILE WIOTH A KEY         *
007300*                             > OR            = THE        KEY   *
007400*                             CONTAINED IN 'MMEXIO-KEY'.         *
007500*    MMEXRB-START-EQUAL     - START THE FILE WITH A KEY =        *
007600*                             TO THE KEY CONTAINED IN            *
007700*                             'MMEXIO-KEY'.                      *
007800*    MMEXRB-READ-SEQ        - READ THE NEXT SEQUENTIAL RECORD    *
007900*                             IN THE FILE.                       *
008000*    MMEXRB-READ-RANDOM     - READ THE RECORD IN THE FILE        *
008100*                             CORRESPONDING TO THE KEY CONTAINED *
008200*                             IN 'MMEXIO-KEY'.                   *
008300*    MMEXRB-WRITE           - WRITE A RECORD (THE RECORD MUST    *
008400*                             BE IN 'MMEXIO-RECORD').            *
008500*    MMEXRB-REWRITE         - REWRITE A RECORD (THE RECORD MUST  *
008600*                             BE IN 'MMEXIO-RECORD').            *
008700*    MMEXRB-DELETE          - DELETE THE RECORD IN THE FILE      *
008800*                             CORRESPONDING TO THE KEY CONTAINED *
008900*                             IN 'MMEXIO-KEY'.                   *
009000*    MMEXRB-CLOSE           - CLOSE THE FILE.                    *
009100*                                                                *
009200* 3. SET ONE OR BOTH OF THE 'TEMPORARY' RETURN CODE FIELDS       *
009300*    'MMEXRB-TEMP-STATUS-1 AND -2'.  THESE TWO RETURN CODES      *
009400*    ARE TO BE USED ON A PER USE BASIS OF THIS ROUTINE.          *
009500*    THESE TWO RETURN CODES ARE INITIALIZED TO ZERO AND ARE      *
009600*    BOTH RESET TO ZERO AFTER EACH CALL OF THIS PROGRAM.         *
009700*    THE TEMPORARY RETURN CODE FIELDS ARE CHECKED FIRST AND      *
009800*    THEN THE PERMANENT RETURN CODES (POINT 1.C ABOVE) WILL      *
009900*    BE CHECKED.                                                 *
010000*                                                                *
010100* 4. PERFORM MB00PD-MMEX-ACCESS THRU MB00PD-MMEX-ACCESS-EXIT.    *
010200*                                                                *
010300*    A. YOU MUST HANDLE MANUALLY ANY RETURN CODE (EITHER TEMP    *
010400*       OR PERM) THAT YOU REQUESTED, AND IF NEEDED, YOUR OWN     *
010500*       ABEND REQUEST.                                           *
010600*                                                                *
010700*    B. ALL ACCESS ROUTINES MOVE THE FILE STATUS TO THE FIELD    *
010800*       SS02WS-FILE-STATUS IN COPYBOOK SS02WS. USE THE '88' LEVEL*
010900*       NAMES TO TEST YOUR MANUAL STATUS CHECKS.  ALSO IN COPY   *
011000*       BOOK SS02WS, USE THE 'WS-' FILE STATUS DATA NAMES WHEN   *
011100*       SETTING THE PERM OR TEMP FILE STATUS CODES PRIOR TO      *
011200*       PERFORMING THE ACCESS ROUTINE.  FOR VSAM FILES, THE VSAM *
011300*       RETURN CODE IS MOVED TO THE FIELD 'SS02WS-FILE-RETURN' IN*
011400*       COPYBOOK SS02WS.                                         *
011500*                                                                *
011600******************************************************************
011700/
011800 ENVIRONMENT DIVISION.
011900*
012000 CONFIGURATION SECTION.
012100 SOURCE-COMPUTER.            IBM-4341.
012200 OBJECT-COMPUTER.            IBM-4341.
012300*
012400 INPUT-OUTPUT SECTION.
012500*
012600 FILE-CONTROL.
012700*
012800     SELECT  MMEX-FILE-SEQ
012900             ASSIGN TO MMEX
013000             ORGANIZATION IS INDEXED
013100             ACCESS       IS SEQUENTIAL
013200             FILE STATUS  IS WS-STATUS
013300                             WS-RETURN
013400             RECORD KEY   IS MMEX-SEQ-KEY.
013500*
013600     SELECT  MMEX-FILE-DYN
013700             ASSIGN TO MMEX
013800             ORGANIZATION IS INDEXED
013900             ACCESS       IS DYNAMIC
014000             FILE STATUS  IS WS-STATUS
014100                             WS-RETURN
014200             RECORD KEY   IS MMEX-DYN-KEY.
014300*
014400     SELECT  MMEX-FILE-RDM
014500             ASSIGN TO MMEX
014600             ORGANIZATION IS INDEXED
014700             ACCESS       IS RANDOM
014800             FILE STATUS  IS WS-STATUS
014900                             WS-RETURN
015000             RECORD KEY   IS MMEX-RDM-KEY.
015100*
015200 I-O-CONTROL.
015300*
015400     SAME AREA FOR
015500         MMEX-FILE-SEQ
015600         MMEX-FILE-DYN
015700         MMEX-FILE-RDM.
015800/
015900 DATA DIVISION.
016000*
016100 FILE SECTION.
016200*
016300***********************
016400*    SEQUENTIAL FD    *
016500***********************
016600*
016700 FD  MMEX-FILE-SEQ
016800     RECORD CONTAINS 16300 CHARACTERS.
016900*
017000 01  MMEX-SEQ-RECORD.
017100     03  MMEX-SEQ-KEY        PIC  X(08).
017200     03  MMEX-SEQ-DATA       PIC  X(16292).
017300*
017400********************
017500*    DYNAMIC FD    *
017600********************
017700*
017800 FD  MMEX-FILE-DYN
017900     RECORD CONTAINS 16300 CHARACTERS.
018000*
018100 01  MMEX-DYN-RECORD.
018200     03  MMEX-DYN-KEY        PIC  X(08).
018300     03  MMEX-DYN-DATA       PIC  X(16292).
018400*
018500*******************
018600*    RANDOM FD    *
018700*******************
018800*
018900 FD  MMEX-FILE-RDM
019000     RECORD CONTAINS 16300 CHARACTERS.
019100*
019200 01  MMEX-RDM-RECORD.
019300     03  MMEX-RDM-KEY        PIC  X(08).
019400     03  MMEX-RDM-DATA       PIC  X(16292).
019500/
019600 WORKING-STORAGE SECTION.
019700*
019800 01  FILLER                  PIC  X(12)      VALUE '** WS-BEG **'.
019900*
020000 01  FILLER                  PIC  X(24)      VALUE
020100     'ACCESS METHOD INDICATOR:'.
020200*
020300 01  WS-ACCESS-METHOD        PIC  X(01)      VALUE SPACE.
020400     88  ACCESS-METHOD-NOT-SET               VALUE SPACE.
020500     88  ACCESS-IS-SEQUENTIAL                VALUE 'S'.
020600     88  ACCESS-IS-DYNAMIC                   VALUE 'D'.
020700     88  ACCESS-IS-RANDOM                    VALUE 'R'.
020800*
020900 01  FILLER                  PIC  X(12)      VALUE
021000     'FILE STATUS:'.
021100*
021200 01  WS-STATUS               PIC  X(02)      VALUE ZERO.
021300*
021400 01  FILLER                  PIC  X(16)      VALUE
021500     'EXTENDED RETURN:'.
021600*
021700 01  WS-RETURN.
021800     03  WS-RETURN-CODE      PIC 9(02)       BINARY
021900                                             VALUE ZERO.
022000     03  WS-FUNCTION-CODE    PIC 9(02)       BINARY
022100                                             VALUE ZERO.
022200     03  WS-FEEDBACK-CODE    PIC 9(02)       BINARY
022300                                             VALUE ZERO.
022400*
022500 01  FILLER                  PIC  X(12)      VALUE
022600     'I-O REQUEST:'.
022700*
022800 01  WS-IO-REQUEST           PIC  9(02)      VALUE ZERO.
022900*
023000 01  FILLER                  PIC  X(28)      VALUE
023100     'PROCESSING SUMMARY COUNTERS:'.
023200*
023300 01  WS-SUMMARY-COUNTERS.
023400     03  WS-SC-READS         PIC S9(09)      PACKED-DECIMAL
023500                                             VALUE ZERO.
023600     03  WS-SC-WRITES        PIC S9(09)      PACKED-DECIMAL
023700                                             VALUE ZERO.
023800     03  WS-SC-REWRITES      PIC S9(09)      PACKED-DECIMAL
023900                                             VALUE ZERO.
024000     03  WS-SC-DELETES       PIC S9(09)      PACKED-DECIMAL
024100                                             VALUE ZERO.
024200*
024300 01  FILLER                  PIC  X(27)      VALUE
024400     'PROCESSING SUMMARY MESSAGE:'.
024500*
024600 01  WS-SUMMARY-MESSAGE.
024700     03  FILLER              PIC  X(10)      VALUE '(PGMSTAT) '.
024800     03  WS-SM-PROG-ID       PIC  X(06)      VALUE SPACES.
024900     03  FILLER              PIC  X(03)      VALUE ' - '.
025000     03  WS-SM-FILE-ID       PIC  X(04)      VALUE 'MMEX'.
025100     03  FILLER              PIC  X(11)      VALUE ' - OPENED: '.
025200     03  WS-SM-ACCESS        PIC  X(03)      VALUE SPACES.
025300     03  FILLER              PIC  X(01)      VALUE SPACE.
025400     03  WS-SM-OPENED        PIC  X(06)      VALUE SPACES.
025500     03  FILLER              PIC  X(01)      VALUE SPACE.
025600     03  WS-SM-READS         PIC  ZZZZZZZZ9.
025700     03  FILLER              PIC  X(08)      VALUE ' READS, '.
025800     03  WS-SM-WRITES        PIC  ZZZZZZZZ9.
025900     03  FILLER              PIC  X(09)      VALUE ' WRITES, '.
026000     03  WS-SM-REWRITES      PIC  ZZZZZZZZ9.
026100     03  FILLER              PIC  X(11)      VALUE ' REWRITES, '.
026200     03  WS-SM-DELETES       PIC  ZZZZZZZZ9.
026300     03  FILLER              PIC  X(08)      VALUE ' DELETES'.
026400*
026500     COPY SS00WS.
026600*
026700 01  FILLER                  PIC  X(12)      VALUE '** WS-END **'.
026800/
026900 LINKAGE SECTION.
027000*
027100****************************
027200*    FILE REQUEST BLOCK    *
027300****************************
027400*
027500     COPY MB00RB REPLACING ==:MB00:== BY ==MMEX==.
027600*
027700******************
027800*    I/O AREA    *
027900******************
028000*
028100     COPY MMEXIO REPLACING ==:MMEX:== BY ==MMEX==.
028200*
028300*********************************
028400*    ABEND ROUTINE WORK AREA    *
028500*********************************
028600*
028700     COPY SS02WS.
028800/
028900 PROCEDURE DIVISION          USING MMEXRB-WORK-AREA,
029000                                   MMEXIO-RECORD,
029100                                   SS02WS-ABEND-WORK-AREA.
029200*
029300 A000-MAIN-LINE-SECTION.
029400*
029500*
029600***************************************************
029700*    SET THE ACCESS METHOD AND ABEND WORK AREA    *
029800*    IF THIS IS THE FIRST CALL OF THIS PROGRAM    *
029900***************************************************
030000*
030100     IF  ACCESS-METHOD-NOT-SET
030200         PERFORM Z000-PROGRAM-INITIALIZATION
030300            THRU Z099-PI-EXIT.
030400*
030500**************************************************************
030600*    PROCESS THE FILE DEPENDING OF THE PROCESSING OPTION.    *
030700**************************************************************
030800*
030900     IF  ACCESS-IS-SEQUENTIAL
031000         PERFORM S000-SEQUENTIAL-ACCESS
031100            THRU S999-SA-EXIT
031200     ELSE
031300         IF  ACCESS-IS-DYNAMIC
031400             PERFORM D000-DYNAMIC-ACCESS
031500                THRU D999-DA-EXIT
031600         ELSE
031700             PERFORM R000-RANDOM-ACCESS
031800                THRU R999-RA-EXIT.
031900*
032000*****************************************************
032100*    MOVE THE STATUS CODE AND RETURN CODE TO        *
032200*    THEIR RESPECTIVE FIELDS IN COPYBOOK SS02WS.    *
032300*****************************************************
032400*
032500     MOVE MMEXRB-STATUS          TO SS02WS-FILE-STATUS.
032600     MOVE MMEXRB-RETURN          TO SS02WS-FILE-RETURN.
032700*
032800 A999-GOBACK.
032900*
033000     GOBACK.
033100/
033200 D000-DYNAMIC-ACCESS.
033300*
033400     GO TO D100-DA-INVALID-REQUEST
033500           D100-DA-INVALID-REQUEST
033600           D100-DA-INVALID-REQUEST
033700           D110-DA-OPEN-INPUT
033800           D120-DA-OPEN-OUTPUT
033900           D130-DA-OPEN-I-O
034000           D100-DA-INVALID-REQUEST
034100           D100-DA-INVALID-REQUEST
034200           D100-DA-INVALID-REQUEST
034300           D140-DA-START-GTE
034400           D150-DA-START-EQUAL
034500           D160-DA-READ-SEQ
034600           D170-DA-READ-RANDOM
034700           D180-DA-WRITE
034800           D190-DA-REWRITE
034900           D200-DA-DELETE
035000           D210-DA-CLOSE
035100           DEPENDING ON MMEXRB-IO-REQUEST.
035200*
035300 D100-DA-INVALID-REQUEST.
035400*
035500     MOVE MMEXRB-IO-REQUEST      TO SS02WS-ABEND-REQUEST.
035600     MOVE '- BAD I-O REQ - MMEX' TO SS02WS-ABENDMSG8.
035700     GO TO Z950-ABORT.
035800*
035900 D110-DA-OPEN-INPUT.
036000*
036100     OPEN INPUT MMEX-FILE-DYN.
036200*
036300     IF  WS-STATUS IS EQUAL TO '97'
036400         MOVE '97'               TO MMEXRB-OPEN-STATUS
036500         MOVE LOW-VALUES         TO WS-RETURN
036600         MOVE ZEROS              TO WS-STATUS.
036700*
036800     MOVE 'DYN'                  TO WS-SM-ACCESS.
036900     MOVE 'INPUT'                TO WS-SM-OPENED.
037000     MOVE '- OPEN  ERROR - MMEX' TO SS02WS-ABENDMSG8.
037100     GO TO D900-DA-CHECK-STATUS.
037200*
037300 D120-DA-OPEN-OUTPUT.
037400*
037500     OPEN OUTPUT MMEX-FILE-DYN.
037600*
037700     IF  WS-STATUS IS EQUAL TO '97'
037800         MOVE '97'               TO MMEXRB-OPEN-STATUS
037900         MOVE LOW-VALUES         TO WS-RETURN
038000         MOVE ZEROS              TO WS-STATUS.
038100*
038200     MOVE 'DYN'                  TO WS-SM-ACCESS.
038300     MOVE 'OUTPUT'               TO WS-SM-OPENED.
038400     MOVE '- OPEN  ERROR - MMEX' TO SS02WS-ABENDMSG8.
038500     GO TO D900-DA-CHECK-STATUS.
038600*
038700 D130-DA-OPEN-I-O.
038800*
038900     OPEN I-O MMEX-FILE-DYN.
039000*
039100     MOVE 'DYN'                  TO WS-SM-ACCESS.
039200     MOVE 'I-O'                  TO WS-SM-OPENED.
039300     MOVE '- OPEN  ERROR - MMEX' TO SS02WS-ABENDMSG8.
039400     GO TO D900-DA-CHECK-STATUS.
039500*
039600 D140-DA-START-GTE.
039700*
039800     MOVE MMEXIO-KEY             TO MMEX-DYN-KEY.
039900     START MMEX-FILE-DYN
040000           KEY IS NOT LESS THAN MMEX-DYN-KEY.
040100*
040200     MOVE '- START ERROR - MMEX' TO SS02WS-ABENDMSG8.
040300     GO TO D900-DA-CHECK-STATUS.
040400*
040500 D150-DA-START-EQUAL.
040600*
040700     MOVE MMEXIO-KEY             TO MMEX-DYN-KEY.
040800     START MMEX-FILE-DYN
040900           KEY IS EQUAL TO MMEX-DYN-KEY.
041000*
041100     MOVE '- START ERROR - MMEX' TO SS02WS-ABENDMSG8.
041200     GO TO D900-DA-CHECK-STATUS.
041300*
041400 D160-DA-READ-SEQ.
041500*
041600     READ MMEX-FILE-DYN NEXT RECORD
041700                                 INTO MMEXIO-RECORD.
041800*
041900     ADD 1                       TO WS-SC-READS.
042000     MOVE '- READ  ERROR - MMEX' TO SS02WS-ABENDMSG8.
042100     GO TO D900-DA-CHECK-STATUS.
042200*
042300 D170-DA-READ-RANDOM.
042400*
042500     MOVE MMEXIO-KEY             TO MMEX-DYN-KEY.
042600     READ MMEX-FILE-DYN          INTO MMEXIO-RECORD.
042700*
042800     ADD 1                       TO WS-SC-READS.
042900     MOVE '- READ  ERROR - MMEX' TO SS02WS-ABENDMSG8.
043000     GO TO D900-DA-CHECK-STATUS.
043100*
043200 D180-DA-WRITE.
043300*
043400     WRITE MMEX-DYN-RECORD       FROM MMEXIO-RECORD.
043500*
043600     ADD 1                       TO WS-SC-WRITES.
043700     MOVE '- WRITE ERROR - MMEX' TO SS02WS-ABENDMSG8.
043800     GO TO D900-DA-CHECK-STATUS.
043900*
044000 D190-DA-REWRITE.
044100*
044200     REWRITE MMEX-DYN-RECORD     FROM MMEXIO-RECORD.
044300*
044400     ADD 1                       TO WS-SC-REWRITES.
044500     MOVE '- REWRT ERROR - MMEX' TO SS02WS-ABENDMSG8.
044600     GO TO D900-DA-CHECK-STATUS.
044700*
044800 D200-DA-DELETE.
044900*
045000     MOVE MMEXIO-KEY             TO MMEX-DYN-KEY.
045100     DELETE MMEX-FILE-DYN RECORD.
045200*
045300     ADD 1                       TO WS-SC-DELETES.
045400     MOVE '- DELET ERROR - MMEX' TO SS02WS-ABENDMSG8.
045500     GO TO D900-DA-CHECK-STATUS.
045600*
045700 D210-DA-CLOSE.
045800*
045900     MOVE SPACE                  TO WS-ACCESS-METHOD.
046000     CLOSE  MMEX-FILE-DYN.
046100*
046200     PERFORM Z100-DISPLAY-THE-SUMMARY
046300        THRU Z199-DTS-EXIT.
046400*
046500     MOVE '- CLOSE ERROR - MMEX' TO SS02WS-ABENDMSG8.
046600     GO TO D900-DA-CHECK-STATUS.
046700*
046800 D900-DA-CHECK-STATUS.
046900*
047000     MOVE WS-STATUS              TO MMEXRB-STATUS.
047100     MOVE WS-RETURN              TO MMEXRB-RETURN.
047200*
047300     IF  MMEXRB-STATUS IS EQUAL  TO MMEXRB-TEMP-STATUS-1   OR
047400         MMEXRB-STATUS IS EQUAL  TO MMEXRB-TEMP-STATUS-2   OR
047500         MMEXRB-STATUS IS EQUAL  TO MMEXRB-PERM-STATUS-1   OR
047600         MMEXRB-STATUS IS EQUAL  TO MMEXRB-PERM-STATUS-2   OR
047700         MMEXRB-STATUS IS EQUAL  TO MMEXRB-PERM-STATUS-3   OR
047800         MMEXRB-STATUS IS EQUAL  TO MMEXRB-PERM-STATUS-4   OR
047900         MMEXRB-STATUS IS EQUAL  TO MMEXRB-PERM-STATUS-5
048000         MOVE ZERO               TO MMEXRB-TEMP-STATUS-1
048100         MOVE ZERO               TO MMEXRB-TEMP-STATUS-2
048200     ELSE
048300         GO TO Z900-ABORT.
048400*
048500 D999-DA-EXIT.
048600     EXIT.
048700/
048800 R000-RANDOM-ACCESS.
048900*
049000     GO TO R100-RA-INVALID-REQUEST
049100           R100-RA-INVALID-REQUEST
049200           R100-RA-INVALID-REQUEST
049300           R100-RA-INVALID-REQUEST
049400           R100-RA-INVALID-REQUEST
049500           R100-RA-INVALID-REQUEST
049600           R110-RA-OPEN-INPUT
049700           R120-RA-OPEN-OUTPUT
049800           R130-RA-OPEN-I-O
049900           R100-RA-INVALID-REQUEST
050000           R100-RA-INVALID-REQUEST
050100           R100-RA-INVALID-REQUEST
050200           R140-RA-READ-RANDOM
050300           R150-RA-WRITE
050400           R160-RA-REWRITE
050500           R170-RA-DELETE
050600           R180-RA-CLOSE
050700           DEPENDING ON MMEXRB-IO-REQUEST.
050800*
050900 R100-RA-INVALID-REQUEST.
051000*
051100     MOVE MMEXRB-IO-REQUEST      TO SS02WS-ABEND-REQUEST.
051200     MOVE '- BAD I-O REQ - MMEX' TO SS02WS-ABENDMSG8.
051300     GO TO Z950-ABORT.
051400*
051500 R110-RA-OPEN-INPUT.
051600*
051700     OPEN INPUT MMEX-FILE-RDM.
051800*
051900     IF  WS-STATUS IS EQUAL TO '97'
052000         MOVE '97'               TO MMEXRB-OPEN-STATUS
052100         MOVE LOW-VALUES         TO WS-RETURN
052200         MOVE ZEROS              TO WS-STATUS.
052300*
052400     MOVE 'RDM'                  TO WS-SM-ACCESS.
052500     MOVE 'INPUT'                TO WS-SM-OPENED.
052600     MOVE '- OPEN  ERROR - MMEX' TO SS02WS-ABENDMSG8.
052700     GO TO R900-RA-CHECK-STATUS.
052800*
052900 R120-RA-OPEN-OUTPUT.
053000*
053100     OPEN OUTPUT MMEX-FILE-RDM.
053200*
053300     IF  WS-STATUS IS EQUAL TO '97'
053400         MOVE '97'               TO MMEXRB-OPEN-STATUS
053500         MOVE LOW-VALUES         TO WS-RETURN
053600         MOVE ZEROS              TO WS-STATUS.
053700*
053800     MOVE 'RDM'                  TO WS-SM-ACCESS.
053900     MOVE 'OUTPUT'               TO WS-SM-OPENED.
054000     MOVE '- OPEN  ERROR - MMEX' TO SS02WS-ABENDMSG8.
054100     GO TO R900-RA-CHECK-STATUS.
054200*
054300 R130-RA-OPEN-I-O.
054400*
054500     OPEN I-O MMEX-FILE-RDM.
054600*
054700     MOVE 'RDM'                  TO WS-SM-ACCESS.
054800     MOVE 'I-O'                  TO WS-SM-OPENED.
054900     MOVE '- OPEN  ERROR - MMEX' TO SS02WS-ABENDMSG8.
055000     GO TO R900-RA-CHECK-STATUS.
055100*
055200 R140-RA-READ-RANDOM.
055300*
055400     MOVE MMEXIO-KEY             TO MMEX-RDM-KEY.
055500     READ MMEX-FILE-RDM          INTO MMEXIO-RECORD.
055600*
055700     ADD 1                       TO WS-SC-READS.
055800     MOVE '- READ  ERROR - MMEX' TO SS02WS-ABENDMSG8.
055900     GO TO R900-RA-CHECK-STATUS.
056000*
056100 R150-RA-WRITE.
056200*
056300     WRITE MMEX-RDM-RECORD       FROM MMEXIO-RECORD.
056400*
056500     ADD 1                       TO WS-SC-WRITES.
056600     MOVE '- WRITE ERROR - MMEX' TO SS02WS-ABENDMSG8.
056700     GO TO R900-RA-CHECK-STATUS.
056800*
056900 R160-RA-REWRITE.
057000*
057100     REWRITE MMEX-RDM-RECORD     FROM MMEXIO-RECORD.
057200*
057300     ADD 1                       TO WS-SC-REWRITES.
057400     MOVE '- REWRT ERROR - MMEX' TO SS02WS-ABENDMSG8.
057500     GO TO R900-RA-CHECK-STATUS.
057600*
057700 R170-RA-DELETE.
057800*
057900     MOVE MMEXIO-KEY             TO MMEX-RDM-KEY.
058000     DELETE MMEX-FILE-RDM RECORD.
058100*
058200     ADD 1                       TO WS-SC-DELETES.
058300     MOVE '- DELET ERROR - MMEX' TO SS02WS-ABENDMSG8.
058400     GO TO R900-RA-CHECK-STATUS.
058500*
058600 R180-RA-CLOSE.
058700*
058800     MOVE SPACE                  TO WS-ACCESS-METHOD.
058900     CLOSE  MMEX-FILE-RDM.
059000*
059100     PERFORM Z100-DISPLAY-THE-SUMMARY
059200        THRU Z199-DTS-EXIT.
059300*
059400     MOVE '- CLOSE ERROR - MMEX' TO SS02WS-ABENDMSG8.
059500     GO TO R900-RA-CHECK-STATUS.
059600*
059700 R900-RA-CHECK-STATUS.
059800*
059900     MOVE WS-STATUS              TO MMEXRB-STATUS.
060000     MOVE WS-RETURN              TO MMEXRB-RETURN.
060100*
060200     IF  MMEXRB-STATUS IS EQUAL  TO MMEXRB-TEMP-STATUS-1   OR
060300         MMEXRB-STATUS IS EQUAL  TO MMEXRB-TEMP-STATUS-2   OR
060400         MMEXRB-STATUS IS EQUAL  TO MMEXRB-PERM-STATUS-1   OR
060500         MMEXRB-STATUS IS EQUAL  TO MMEXRB-PERM-STATUS-2   OR
060600         MMEXRB-STATUS IS EQUAL  TO MMEXRB-PERM-STATUS-3   OR
060700         MMEXRB-STATUS IS EQUAL  TO MMEXRB-PERM-STATUS-4   OR
060800         MMEXRB-STATUS IS EQUAL  TO MMEXRB-PERM-STATUS-5
060900         MOVE ZERO               TO MMEXRB-TEMP-STATUS-1
061000         MOVE ZERO               TO MMEXRB-TEMP-STATUS-2
061100     ELSE
061200         GO TO Z900-ABORT.
061300*
061400 R999-RA-EXIT.
061500     EXIT.
061600/
061700 S000-SEQUENTIAL-ACCESS.
061800*
061900     GO TO S110-SA-OPEN-INPUT
062000           S120-SA-OPEN-OUTPUT
062100           S130-SA-OPEN-I-O
062200           S100-SA-INVALID-REQUEST
062300           S100-SA-INVALID-REQUEST
062400           S100-SA-INVALID-REQUEST
062500           S100-SA-INVALID-REQUEST
062600           S100-SA-INVALID-REQUEST
062700           S100-SA-INVALID-REQUEST
062800           S140-SA-START-GTE
062900           S150-SA-START-EQUAL
063000           S160-SA-READ-SEQ
063100           S100-SA-INVALID-REQUEST
063200           S170-SA-WRITE
063300           S180-SA-REWRITE
063400           S190-SA-DELETE
063500           S200-SA-CLOSE
063600           DEPENDING ON MMEXRB-IO-REQUEST.
063700*
063800 S100-SA-INVALID-REQUEST.
063900*
064000     MOVE MMEXRB-IO-REQUEST      TO SS02WS-ABEND-REQUEST.
064100     MOVE '- BAD I-O REQ - MMEX' TO SS02WS-ABENDMSG8.
064200     GO TO Z950-ABORT.
064300*
064400 S110-SA-OPEN-INPUT.
064500*
064600     OPEN INPUT MMEX-FILE-SEQ.
064700*
064800     IF  WS-STATUS IS EQUAL TO '97'
064900         MOVE '97'               TO MMEXRB-OPEN-STATUS
065000         MOVE LOW-VALUES         TO WS-RETURN
065100         MOVE ZEROS              TO WS-STATUS.
065200*
065300     MOVE 'SEQ'                  TO WS-SM-ACCESS.
065400     MOVE 'INPUT'                TO WS-SM-OPENED.
065500     MOVE '- OPEN  ERROR - MMEX' TO SS02WS-ABENDMSG8.
065600     GO TO S900-SA-CHECK-STATUS.
065700*
065800 S120-SA-OPEN-OUTPUT.
065900*
066000     OPEN OUTPUT MMEX-FILE-SEQ.
066100*
066200     IF  WS-STATUS IS EQUAL TO '97'
066300         MOVE '97'               TO MMEXRB-OPEN-STATUS
066400         MOVE LOW-VALUES         TO WS-RETURN
066500         MOVE ZEROS              TO WS-STATUS.
066600*
066700     MOVE 'SEQ'                  TO WS-SM-ACCESS.
066800     MOVE 'OUTPUT'               TO WS-SM-OPENED.
066900     MOVE '- OPEN  ERROR - MMEX' TO SS02WS-ABENDMSG8.
067000     GO TO S900-SA-CHECK-STATUS.
067100*
067200 S130-SA-OPEN-I-O.
067300*
067400     OPEN I-O MMEX-FILE-SEQ.
067500*
067600     MOVE 'SEQ'                  TO WS-SM-ACCESS.
067700     MOVE 'I-O'                  TO WS-SM-OPENED.
067800     MOVE '- OPEN  ERROR - MMEX' TO SS02WS-ABENDMSG8.
067900     GO TO S900-SA-CHECK-STATUS.
068000*
068100 S140-SA-START-GTE.
068200*
068300     MOVE MMEXIO-KEY             TO MMEX-SEQ-KEY.
068400     START MMEX-FILE-SEQ
068500           KEY IS NOT LESS THAN MMEX-SEQ-KEY.
068600*
068700     MOVE '- START ERROR - MMEX' TO SS02WS-ABENDMSG8.
068800     GO TO S900-SA-CHECK-STATUS.
068900*
069000 S150-SA-START-EQUAL.
069100*
069200     MOVE MMEXIO-KEY             TO MMEX-SEQ-KEY.
069300     START MMEX-FILE-SEQ
069400           KEY IS EQUAL TO MMEX-SEQ-KEY.
069500*
069600     MOVE '- START ERROR - MMEX' TO SS02WS-ABENDMSG8.
069700     GO TO S900-SA-CHECK-STATUS.
069800*
069900 S160-SA-READ-SEQ.
070000*
070100     READ MMEX-FILE-SEQ          INTO MMEXIO-RECORD.
070200*
070300     ADD 1                       TO WS-SC-READS.
070400     MOVE '- READ  ERROR - MMEX' TO SS02WS-ABENDMSG8.
070500     GO TO S900-SA-CHECK-STATUS.
070600*
070700 S170-SA-WRITE.
070800*
070900     WRITE MMEX-SEQ-RECORD       FROM MMEXIO-RECORD.
071000*
071100     ADD 1                       TO WS-SC-WRITES.
071200     MOVE '- WRITE ERROR - MMEX' TO SS02WS-ABENDMSG8.
071300     GO TO S900-SA-CHECK-STATUS.
071400*
071500 S180-SA-REWRITE.
071600*
071700     REWRITE MMEX-SEQ-RECORD     FROM MMEXIO-RECORD.
071800*
071900     ADD 1                       TO WS-SC-REWRITES.
072000     MOVE '- REWRT ERROR - MMEX' TO SS02WS-ABENDMSG8.
072100     GO TO S900-SA-CHECK-STATUS.
072200*
072300 S190-SA-DELETE.
072400*
072500     MOVE MMEXIO-KEY             TO MMEX-SEQ-KEY.
072600     DELETE MMEX-FILE-SEQ RECORD.
072700*
072800     ADD 1                       TO WS-SC-DELETES.
072900     MOVE '- DELET ERROR - MMEX' TO SS02WS-ABENDMSG8.
073000     GO TO S900-SA-CHECK-STATUS.
073100*
073200 S200-SA-CLOSE.
073300*
073400     MOVE SPACE                  TO WS-ACCESS-METHOD.
073500     CLOSE MMEX-FILE-SEQ.
073600*
073700     PERFORM Z100-DISPLAY-THE-SUMMARY
073800        THRU Z199-DTS-EXIT.
073900*
074000     MOVE '- CLOSE ERROR - MMEX' TO SS02WS-ABENDMSG8.
074100     GO TO S900-SA-CHECK-STATUS.
074200*
074300 S900-SA-CHECK-STATUS.
074400*
074500     MOVE WS-STATUS              TO MMEXRB-STATUS.
074600     MOVE WS-RETURN              TO MMEXRB-RETURN.
074700*
074800     IF  MMEXRB-STATUS IS EQUAL  TO MMEXRB-TEMP-STATUS-1   OR
074900         MMEXRB-STATUS IS EQUAL  TO MMEXRB-TEMP-STATUS-2   OR
075000         MMEXRB-STATUS IS EQUAL  TO MMEXRB-PERM-STATUS-1   OR
075100         MMEXRB-STATUS IS EQUAL  TO MMEXRB-PERM-STATUS-2   OR
075200         MMEXRB-STATUS IS EQUAL  TO MMEXRB-PERM-STATUS-3   OR
075300         MMEXRB-STATUS IS EQUAL  TO MMEXRB-PERM-STATUS-4   OR
075400         MMEXRB-STATUS IS EQUAL  TO MMEXRB-PERM-STATUS-5
075500         MOVE ZERO               TO MMEXRB-TEMP-STATUS-1
075600         MOVE ZERO               TO MMEXRB-TEMP-STATUS-2
075700     ELSE
075800         GO TO Z900-ABORT.
075900*
076000 S999-SA-EXIT.
076100     EXIT.
076200/
076300 Z000-PROGRAM-INITIALIZATION.
076400*
076500************************************************************
076600*    THE CALLING PROGRAM MUST SUPPLY THE PROGRAM ID TO     *
076700*    ENSURE THAT ANY SUBSEQUENT ABEND CAN BE TRACED TO     *
076800*    IT.  IF THE PROGRAM ID IS NOT SUPPLIED, THE ACCESS    *
076900*    ROUTINE ABENDS HERE.                                  *
077000************************************************************
077100*
077200     IF  SS02WS-ABEND-PROG-ID IS EQUAL TO SPACES
077300         MOVE '- BAD PROG NAME - MMEX'
077400                                 TO SS02WS-ABENDMSG8
077500         MOVE 'BADPRO'           TO SS02WS-ABEND-PROG-ID
077600         GO TO Z950-ABORT.
077700*
077800     MOVE SS02WS-ABEND-PROG-ID   TO WS-SM-PROG-ID.
077900*
078000*************************************************************
078100*    SINCE THIS ROUTINE IS PERFORMED ONLY ONCE DURING       *
078200*    THE EXECUTION OF A CALLING PROGRAM, THE FIRST          *
078300*    REQUEST MUST BE FOR AN OPEN.  SET THE ACCESS METHOD    *
078400*    (SEQENTIAL, DYNAMIC, OR RANDOM), OR ISSUE AN ABEND     *
078500*    IF THE REQUEST IS NOT FOR AN OPEN.                     *
078600*************************************************************
078700*
078800     IF  MMEXRB-OPEN-INPUT    OR
078900         MMEXRB-OPEN-OUTPUT   OR
079000         MMEXRB-OPEN-IO
079100         MOVE 'S'                TO WS-ACCESS-METHOD.
079200*
079300     IF  MMEXRB-OPEN-INPUT-DYN    OR
079400         MMEXRB-OPEN-OUTPUT-DYN   OR
079500         MMEXRB-OPEN-IO-DYN
079600         MOVE 'D'                TO WS-ACCESS-METHOD.
079700*
079800     IF  MMEXRB-OPEN-INPUT-RDM    OR
079900         MMEXRB-OPEN-OUTPUT-RDM   OR
080000         MMEXRB-OPEN-IO-RDM
080100         MOVE 'R'                TO WS-ACCESS-METHOD.
080200*
080300     IF  ACCESS-METHOD-NOT-SET
080400         MOVE 'SMMEX'            TO SS02WS-ABEND-CODE-X
080500         MOVE MMEXRB-IO-REQUEST  TO WS-IO-REQUEST
080600         MOVE WS-IO-REQUEST      TO SS02WS-ABEND-REQUEST
080700         MOVE 'ILLEGAL ACCESS REQUEST'
080800                                 TO SS02WS-ABENDMSG8
080900         GO TO Z950-ABORT.
081000*
081100 Z099-PI-EXIT.
081200     EXIT.
081300*
081400 Z100-DISPLAY-THE-SUMMARY.
081500*
081600     MOVE WS-SC-READS            TO WS-SM-READS.
081700     MOVE WS-SC-WRITES           TO WS-SM-WRITES.
081800     MOVE WS-SC-REWRITES         TO WS-SM-REWRITES.
081900     MOVE WS-SC-DELETES          TO WS-SM-DELETES.
082000     DISPLAY WS-SUMMARY-MESSAGE.
082100*
082200     INITIALIZE WS-SUMMARY-COUNTERS.
082300*
082400 Z199-DTS-EXIT.
082500     EXIT.
082600/
082700 Z900-ABORT.
082800*
082900     MOVE 'SMMEX'                TO SS02WS-ABEND-CODE-X.
083000     MOVE MMEXRB-STATUS          TO SS02WS-ABEND-STATUS.
083100     MOVE MMEXRB-RETURN-CODE     TO SS02WS-ABEND-RETURN.
083200     MOVE MMEXRB-FUNCTION-CODE   TO SS02WS-ABEND-FUNCTION.
083300     MOVE MMEXRB-FEEDBACK-CODE   TO SS02WS-ABEND-FEEDBACK.
083400     MOVE MMEXRB-IO-REQUEST      TO WS-IO-REQUEST.
083500     MOVE WS-IO-REQUEST          TO SS02WS-ABEND-REQUEST.
083600*
083700 Z950-ABORT.
083800*
083900     PERFORM SS02PD-ABEND        THRU SS02PD-ABEND-EXIT.
084000*
084100 Z999-ABORT-EXIT.
084200     EXIT.
084300/
084400********************************
084500*    STANDARD ABEND ROUTINE    *
084600********************************
084700*
084800     COPY SS02PD.
084900*
