000100 IDENTIFICATION DIVISION.
000200 PROGRAM-ID. MTL1AB.
000201*       RELEASE DATE=08/24/01.        REVISION LEVEL=08.06      * ++++++++
000202***************************************************************** ++++++++
000203*WARNING.  THIS COMPUTER PROGRAM IS PROTECTED BY COPYRIGHT LAW  * ++++++++
000204*AND INTERNATIONAL TREATIES.  UNAUTHORIZED REPRODUCTION OR      * ++++++++
000205*DISTRIBUTION OF THIS PROGRAM, OR ANY PORTION OF IT, MAY RESULT * ++++++++
000206*IN SEVERE CIVIL AND CRIMINAL PENATLIES, AND WILL BE PROSECUTED * ++++++++
000207*TO THE MAXIMUM EXTENT POSSIBLE UNDER THE LAW.                  * ++++++++
000208*COPYRIGHT 1999 PAYSYS INTERNATIONAL, INC.  ALL RIGHTS RESERVED.* ++++++++
000209***************************************************************** ++++++++
000300*
000400******************************************************************
000500*                                                                *
000600*    ---------------MERCHANT BANKCARD SYSTEM---------------      *
000700*                                                                *
000800*          MTL1AB - MTL1 FILE COMMON BATCH I/O ROUTINE           *
000900*                                                                *
001000*                VARIABLE LENGTH SEQUENTIAL FILE                 *
001100*                                                                *
001200*    ----------------TEMPORARY LOG FILE 1------------------      *
001300*                                                                *
001400******************************************************************
001500*                                                                *
001600*    COPYBOOKS WHICH MUST APPEAR IN YOUR BATCH PROGRAM:          *
001700*                                                                *
001800*    (WS) MB00RB - BATCH REQUEST BLOCK                           *
001900*    (WS) MML1IO - I/O AREA                                      *
002000*    (WS) SS02WS - STANDARD ABEND ROUTINE WORK AREA              *
002100*    (PD) MB00PD - PROCEDURE DIVISION CALL STATEMENT             *
002200*                                                                *
002300******************************************************************
002400*                                                                *
002500*    TO USE THIS ROUTINE.........                                *
002600*                                                                *
002700* 1. HOUSEKEEPING DUTIES:                                        *
002800*                                                                *
002900*    A. IN THE HOUSEKEEPING SECTION OF YOUR PROGRAM, MOVE THE    *
003000*       NAME OF THE PROGRAM USING THIS ROUTINE TO THE FIELD      *
003100*       'SS02WS-ABEND-PROG-ID'. THIS FIELD IS USED AS PART OF THE*
003200*       STANDARD ABEND MESSAGES IF A PROGRAM ABORTS WITHIN THIS  *
003300*       ACCESS ROUTINE.  IF THE FIELD 'SS02WS-ABEND-PROG-ID' IS  *
003400*       LEFT BLANK, THIS ROUTINE WILL FORCE AN ABEND SITUATION.  *
003500*                                                                *
003600*    B. BEFORE PERFORMING THE ACCESS ROUTINE, MOVE A LITERAL TO  *
003700*       THE FIELD 'SS02WS-ABEND-LOCATION'. THIS FIELD CAN BE USED*
003800*       TO INDICATE WHERE THE ROUTINE IS BEING 'FROM' IN CASE    *
003900*       OF ABNORMAL TERMINATION. (E.G. DEBUGGING ASSISTANCE)     *
004000*                                                                *
004100*    C. SET THE RETURN CODES WHICH YOU WANT, FOR THE DURATION    *
004200*       OF THIS PROGRAM, TO BE CONSIDERED VALID IN THE FIELDS    *
004300*       'MTL1RB-PERM-STATUS-1 TO -5'. THESE FIELDS ARE INITIAL-  *
004400*       IZED TO ZERO (AS ZERO IS ALWAYS AN ACCEPTABLE RETURN     *
004500*       CODE).  THIS ROUTINE WILL (IN ALL I-O REQUESTS) SUCCESS- *
004600*       FULLY SATISFY YOUR PERFORM REQUEST IF ANY OF THE REQUESTS*
004700*       ENCOUNTER ANY OF THE PERMANENT STATUS CODES 1-5.         *
004800*                                                                *
004900* 2. SET ONE OF THE FOLLOWING CONDITION NAMES TO 'TRUE' TO SET   *
005000*    THE FILE REQUEST:                                           *
005100*                                                                *
005200*        CONDITION NAME                EXPLANATION               *
005300*   *----------------------***---------------------------------* *
005400*                                                                *
005500*    MTL1RB-OPEN-INPUT      - OPEN THE FILE AS INPUT.            *
005600*    MTL1RB-OPEN-OUTPUT     - OPEN THE FILE AS OUTPUT.           *
005700*    MTL1RB-OPEN-IO         - OPEN THE FILE AS I-O.              *
005800*    MTL1RB-READ-SEQ        - READ THE NEXT SEQUENTIAL RECORD    *
005900*                             IN THE FILE.                       *
006000*    MTL1RB-WRITE           - WRITE A RECORD (THE RECORD MUST    *
006100*                             BE IN 'MTL1IO-RECORD').            *
006200*    MTL1RB-REWRITE         - REWRITE A RECORD (THE RECORD MUST  *
006300*                             BE IN 'MTL1IO-RECORD').            *
006400*    MTL1RB-CLOSE           - CLOSE THE FILE.                    *
006500*                                                                *
006600* 3. SET ONE OR BOTH OF THE 'TEMPORARY' RETURN CODE FIELDS       *
006700*    'MTL1RB-TEMP-STATUS-1 AND -2'.  THESE TWO RETURN CODES      *
006800*    ARE TO BE USED ON A PER USE BASIS OF THIS ROUTINE.          *
006900*    THESE TWO RETURN CODES ARE INITIALIZED TO ZERO AND ARE      *
007000*    BOTH RESET TO ZERO AFTER EACH CALL OF THIS PROGRAM.         *
007100*    THE TEMPORARY RETURN CODE FIELDS ARE CHECKED FIRST AND      *
007200*    THEN THE PERMANENT RETURN CODES (POINT 1.C ABOVE) WILL      *
007300*    BE CHECKED.                                                 *
007400*                                                                *
007500* 4. PERFORM MB00PD-MTL1-ACCESS THRU MB00PD-MTL1-ACCESS-EXIT.    *
007600*                                                                *
007700*    A. YOU MUST HANDLE MANUALLY ANY RETURN CODE (EITHER TEMP    *
007800*       OR PERM) THAT YOU REQUESTED, AND IF NEEDED, YOUR OWN     *
007900*       ABEND REQUEST.                                           *
008000*                                                                *
008100*    B. ALL ACCESS ROUTINES MOVE THE FILE STATUS TO THE FIELD    *
008200*       SS02WS-FILE-STATUS IN COPYBOOK SS02WS. USE THE '88' LEVEL*
008300*       NAMES TO TEST YOUR MANUAL STATUS CHECKS.  ALSO IN COPY   *
008400*       BOOK SS02WS, USE THE 'WS-' FILE STATUS DATA NAMES WHEN   *
008500*       SETTING THE PERM OR TEMP FILE STATUS CODES PRIOR TO      *
008600*       PERFORMING THE ACCESS ROUTINE.                           *
008700*                                                                *
008800******************************************************************
008900/
009000 ENVIRONMENT DIVISION.
009100*
009200 CONFIGURATION SECTION.
009300 SOURCE-COMPUTER.            IBM-4341.
009400 OBJECT-COMPUTER.            IBM-4341.
009500*
009600 INPUT-OUTPUT SECTION.
009700*
009800 FILE-CONTROL.
009900*
010000     SELECT  MTL1-FILE
010100             ASSIGN              TO SYS002-UT-3380-S-MTL1
010200             FILE STATUS         IS WS-STATUS.
010300/
010400 DATA DIVISION.
010500*
010600 FILE SECTION.
010700*
010800 FD  MTL1-FILE
010900     RECORDING MODE IS V
011000     RECORD CONTAINS 74 TO 16374 CHARACTERS
011100     BLOCK CONTAINS 0 CHARACTERS.
011200*
011300 01  MTL1-RECORD.
011400     03  MTL1-FD-KEY-1       PIC  X(14).
011500     03  MTL1-FD-KEY-2       PIC  X(60).
011600     03  MTL1-DATA           PIC  X(01)      OCCURS
011700                                             0 TO 16300 TIMES
011800                                             DEPENDING ON
011900                                             WS-DATA-LENGTH.
012000*
012100/
012200 WORKING-STORAGE SECTION.
012300*
012400 01  FILLER                  PIC  X(12)      VALUE '** WS-BEG **'.
012500*
012600 01  FILLER                  PIC  X(18)      VALUE
012700     'FIRST CALL SWITCH:'.
012800*
012900 01  WS-FIRST-CALL-SW        PIC  X(01)      VALUE 'Y'.
013000     88  WS-FIRST-CALL-SW-IS-YES             VALUE 'Y'.
013100*
013200 01  FILLER                  PIC  X(12)      VALUE
013300     'FILE STATUS:'.
013400*
013500 01  WS-STATUS               PIC  X(02)      VALUE ZERO.
013600*
013700 01  FILLER                  PIC  X(12)      VALUE
013800     'I-O REQUEST:'.
013900*
014000 01  WS-IO-REQUEST           PIC  9(02)      VALUE ZERO.
014100*
014200 01  FILLER                  PIC  X(12)      VALUE
014300     'DATA LENGTH:'.
014400*
014500 01  WS-DATA-LENGTH          PIC S9(04)      BINARY
014600                                             VALUE ZERO.
014700*
014800 01  FILLER                  PIC  X(28)      VALUE
014900     'PROCESSING SUMMARY COUNTERS:'.
015000*
015100 01  WS-SUMMARY-COUNTERS.
015200     03  WS-SC-READS         PIC S9(09)      PACKED-DECIMAL
015300                                             VALUE ZERO.
015400     03  WS-SC-WRITES        PIC S9(09)      PACKED-DECIMAL
015500                                             VALUE ZERO.
015600     03  WS-SC-REWRITES      PIC S9(09)      PACKED-DECIMAL
015700                                             VALUE ZERO.
015800*
015900 01  FILLER                  PIC  X(27)      VALUE
016000     'PROCESSING SUMMARY MESSAGE:'.
016100*
016200 01  WS-SUMMARY-MESSAGE.
016300     03  FILLER              PIC  X(10)      VALUE '(PGMSTAT) '.
016400     03  WS-SM-PROG-ID       PIC  X(06)      VALUE SPACES.
016500     03  FILLER              PIC  X(03)      VALUE ' - '.
016600     03  WS-SM-FILE-ID       PIC  X(04)      VALUE 'MTL1'.
016700     03  FILLER              PIC  X(11)      VALUE ' - OPENED: '.
016800     03  WS-SM-ACCESS        PIC  X(03)      VALUE SPACES.
016900     03  FILLER              PIC  X(01)      VALUE SPACE.
017000     03  WS-SM-OPENED        PIC  X(06)      VALUE SPACES.
017100     03  FILLER              PIC  X(01)      VALUE SPACE.
017200     03  WS-SM-READS         PIC  ZZZZZZZZ9.
017300     03  FILLER              PIC  X(08)      VALUE ' READS, '.
017400     03  WS-SM-WRITES        PIC  ZZZZZZZZ9.
017500     03  FILLER              PIC  X(09)      VALUE ' WRITES, '.
017600     03  WS-SM-REWRITES      PIC  ZZZZZZZZ9.
017700     03  FILLER              PIC  X(09)      VALUE ' REWRITES'.
017800/
017900     COPY SS00WS.
018000*
018100 01  FILLER                  PIC  X(12)      VALUE '** WS-END **'.
018200/
018300 LINKAGE SECTION.
018400*
018500****************************
018600*    FILE REQUEST BLOCK    *
018700****************************
018800*
018900     COPY MB00RB REPLACING ==:MB00:== BY ==MTL1==.
019000*
019100******************
019200*    I/O AREA    *
019300******************
019400*
019500     COPY MML1IO REPLACING ==:MML1:== BY ==MTL1==.
019600*
019700*********************************
019800*    ABEND ROUTINE WORK AREA    *
019900*********************************
020000*
020100     COPY SS02WS.
020200/
020300 PROCEDURE DIVISION          USING MTL1RB-WORK-AREA,
020400                                   MTL1IO-RECORD,
020500                                   SS02WS-ABEND-WORK-AREA.
020600*
020700 A000-MAIN-LINE-SECTION.
020800*
020900*
021000******************************************
021100*    SET THE ABEND WORK AREA IF THIS     *
021200*    IS THE FIRST CALL OF THE PROGRAM    *
021300******************************************
021400*
021500     IF  WS-FIRST-CALL-SW-IS-YES
021600         PERFORM Z000-PROGRAM-INITIALIZATION
021700            THRU Z099-PI-EXIT.
021800*
021900**************************
022000*    PROCESS THE FILE    *
022100**************************
022200*
022300     PERFORM S000-SEQUENTIAL-ACCESS
022400        THRU S999-SA-EXIT.
022500*
022600************************************************
022700*    MOVE THE STATUS CODE TO ITS RESPECTIVE    *
022800*    FIELD IN COPYBOOK SS02WS.                 *
022900************************************************
023000*
023100     MOVE MTL1RB-STATUS          TO SS02WS-FILE-STATUS.
023200*
023300 A999-GOBACK.
023400*
023500     GOBACK.
023600/
023700 S000-SEQUENTIAL-ACCESS.
023800*
023900     GO TO S110-SA-OPEN-INPUT
024000           S120-SA-OPEN-OUTPUT
024100           S130-SA-OPEN-I-O
024200           S100-SA-INVALID-REQUEST
024300           S100-SA-INVALID-REQUEST
024400           S100-SA-INVALID-REQUEST
024500           S100-SA-INVALID-REQUEST
024600           S100-SA-INVALID-REQUEST
024700           S100-SA-INVALID-REQUEST
024800           S100-SA-INVALID-REQUEST
024900           S100-SA-INVALID-REQUEST
025000           S140-SA-READ-SEQ
025100           S100-SA-INVALID-REQUEST
025200           S150-SA-WRITE
025300           S160-SA-REWRITE
025400           S100-SA-INVALID-REQUEST
025500           S170-SA-CLOSE
025600           DEPENDING ON MTL1RB-IO-REQUEST.
025700*
025800 S100-SA-INVALID-REQUEST.
025900*
026000     MOVE MTL1RB-IO-REQUEST      TO SS02WS-ABEND-REQUEST.
026100     MOVE '- BAD I-O REQ - MTL1' TO SS02WS-ABENDMSG8.
026200     GO TO Z950-ABORT.
026300*
026400 S110-SA-OPEN-INPUT.
026500*
026600     OPEN INPUT MTL1-FILE.
026700*
026800     MOVE 'SEQ'                  TO WS-SM-ACCESS.
026900     MOVE 'INPUT'                TO WS-SM-OPENED.
027000     MOVE '- OPEN  ERROR - MTL1' TO SS02WS-ABENDMSG8.
027100     GO TO S900-SA-CHECK-STATUS.
027200*
027300 S120-SA-OPEN-OUTPUT.
027400*
027500     OPEN OUTPUT MTL1-FILE.
027600*
027700     MOVE 'SEQ'                  TO WS-SM-ACCESS.
027800     MOVE 'OUTPUT'               TO WS-SM-OPENED.
027900     MOVE '- OPEN  ERROR - MTL1' TO SS02WS-ABENDMSG8.
028000     GO TO S900-SA-CHECK-STATUS.
028100*
028200 S130-SA-OPEN-I-O.
028300*
028400     OPEN I-O MTL1-FILE.
028500*
028600     MOVE 'SEQ'                  TO WS-SM-ACCESS.
028700     MOVE 'I-O'                  TO WS-SM-OPENED.
028800     MOVE '- OPEN  ERROR - MTL1' TO SS02WS-ABENDMSG8.
028900     GO TO S900-SA-CHECK-STATUS.
029000*
029100 S140-SA-READ-SEQ.
029200*
029300     READ MTL1-FILE              INTO MTL1IO-RECORD.
029400*
029500     ADD 1                       TO WS-SC-READS.
029600     MOVE '- READ  ERROR - MTL1' TO SS02WS-ABENDMSG8.
029700     GO TO S900-SA-CHECK-STATUS.
029800*
029900 S150-SA-WRITE.
030000*
030100     MOVE MTL1IO-VARIABLE-LENGTH
030200                                 TO WS-DATA-LENGTH.
030300*
030400     WRITE MTL1-RECORD           FROM MTL1IO-RECORD.
030500*
030600     ADD 1                       TO WS-SC-WRITES.
030700     MOVE '- WRITE ERROR - MTL1' TO SS02WS-ABENDMSG8.
030800     GO TO S900-SA-CHECK-STATUS.
030900*
031000 S160-SA-REWRITE.
031100*
031200     MOVE MTL1IO-VARIABLE-LENGTH
031300                                 TO WS-DATA-LENGTH.
031400*
031500     REWRITE MTL1-RECORD         FROM MTL1IO-RECORD.
031600*
031700     ADD 1                       TO WS-SC-REWRITES.
031800     MOVE '- REWRT ERROR - MTL1' TO SS02WS-ABENDMSG8.
031900     GO TO S900-SA-CHECK-STATUS.
032000*
032100 S170-SA-CLOSE.
032200*
032300     MOVE SPACE                  TO WS-FIRST-CALL-SW.
032400     CLOSE MTL1-FILE.
032500*
032600     PERFORM Z100-DISPLAY-THE-SUMMARY
032700        THRU Z199-DTS-EXIT.
032800*
032900     MOVE '- CLOSE ERROR - MTL1' TO SS02WS-ABENDMSG8.
033000     GO TO S900-SA-CHECK-STATUS.
033100*
033200 S900-SA-CHECK-STATUS.
033300*
033400     MOVE WS-STATUS              TO MTL1RB-STATUS.
033500*
033600     IF  MTL1RB-STATUS IS EQUAL  TO MTL1RB-TEMP-STATUS-1   OR
033700         MTL1RB-STATUS IS EQUAL  TO MTL1RB-TEMP-STATUS-2   OR
033800         MTL1RB-STATUS IS EQUAL  TO MTL1RB-PERM-STATUS-1   OR
033900         MTL1RB-STATUS IS EQUAL  TO MTL1RB-PERM-STATUS-2   OR
034000         MTL1RB-STATUS IS EQUAL  TO MTL1RB-PERM-STATUS-3   OR
034100         MTL1RB-STATUS IS EQUAL  TO MTL1RB-PERM-STATUS-4   OR
034200         MTL1RB-STATUS IS EQUAL  TO MTL1RB-PERM-STATUS-5
034300         MOVE ZERO               TO MTL1RB-TEMP-STATUS-1
034400         MOVE ZERO               TO MTL1RB-TEMP-STATUS-2
034500     ELSE
034600         GO TO Z900-ABORT.
034700*
034800 S999-SA-EXIT.
034900     EXIT.
035000/
035100 Z000-PROGRAM-INITIALIZATION.
035200*
035300************************************************************
035400*    THE CALLING PROGRAM MUST SUPPLY THE PROGRAM ID TO     *
035500*    ENSURE THAT ANY SUBSEQUENT ABEND CAN BE TRACED TO     *
035600*    IT.  IF THE PROGRAM ID IS NOT SUPPLIED, THE ACCESS    *
035700*    ROUTINE ABENDS HERE.                                  *
035800************************************************************
035900*
036000     IF  SS02WS-ABEND-PROG-ID IS EQUAL TO SPACES
036100         MOVE '- BAD PROG NAME - MTL1'
036200                                 TO SS02WS-ABENDMSG8
036300         MOVE 'BADPRO'           TO SS02WS-ABEND-PROG-ID
036400         GO TO Z950-ABORT.
036500*
036600     MOVE SS02WS-ABEND-PROG-ID   TO WS-SM-PROG-ID.
036700*
036800*************************************************************
036900*    SINCE THIS ROUTINE IS PERFORMED ONLY ONCE DURING       *
037000*    THE EXECUTION OF A CALLING PROGRAM, THE FIRST          *
037100*    REQUEST MUST BE FOR AN OPEN.  SET THE ACCESS METHOD    *
037200*    (SEQENTIAL, DYNAMIC, OR RANDOM), OR ISSUE AN ABEND     *
037300*    IF THE REQUEST IS NOT FOR AN OPEN.                     *
037400*************************************************************
037500*
037600     IF  MTL1RB-OPEN-INPUT    OR
037700         MTL1RB-OPEN-OUTPUT   OR
037800         MTL1RB-OPEN-IO
037900         MOVE 'N'                TO WS-FIRST-CALL-SW
038000     ELSE
038100         MOVE 'ILLEGAL ACCESS REQUEST'
038200                                 TO SS02WS-ABENDMSG8
038300         GO TO Z950-ABORT.
038400*
038500 Z099-PI-EXIT.
038600     EXIT.
038700*
038800 Z100-DISPLAY-THE-SUMMARY.
038900*
039000     MOVE WS-SC-READS            TO WS-SM-READS.
039100     MOVE WS-SC-WRITES           TO WS-SM-WRITES.
039200     MOVE WS-SC-REWRITES         TO WS-SM-REWRITES.
039300     DISPLAY WS-SUMMARY-MESSAGE.
039400*
039500     INITIALIZE WS-SUMMARY-COUNTERS.
039600*
039700 Z199-DTS-EXIT.
039800     EXIT.
039900/
040000 Z900-ABORT.
040100*
040200     MOVE 'SMTL1'                TO SS02WS-ABEND-CODE-X.
040300     MOVE MTL1RB-STATUS          TO SS02WS-ABEND-STATUS.
040400     MOVE MTL1RB-IO-REQUEST      TO WS-IO-REQUEST.
040500     MOVE WS-IO-REQUEST          TO SS02WS-ABEND-REQUEST.
040600*
040700 Z950-ABORT.
040800*
040900     PERFORM SS02PD-ABEND        THRU SS02PD-ABEND-EXIT.
041000*
041100 Z999-ABORT-EXIT.
041200     EXIT.
041300/
041400********************************
041500*    STANDARD ABEND ROUTINE    *
041600********************************
041700*
041800     COPY SS02PD.
041900*
